import threading
from typing import AbstractSet, Optional

from .runnable import GraphRunnableTask
from .base import BaseRunner

from dbt.contracts.graph.manifest import WritableManifest
from dbt.contracts.results import RunStatus, RunResult
from dbt.exceptions import DbtInternalError, DbtRuntimeError
from dbt.graph import ResourceTypeSelector
from dbt.events.functions import fire_event
from dbt.events.types import CompileComplete
from dbt.node_types import NodeType


class CompileRunner(BaseRunner):
    def before_execute(self):
        pass

    def after_execute(self, result):
        pass

    def execute(self, compiled_node, manifest):
        return RunResult(
            node=compiled_node,
            status=RunStatus.Success,
            timing=[],
            thread_id=threading.current_thread().name,
            execution_time=0,
            message=None,
            adapter_response={},
            failures=None,
        )

    def compile(self, manifest):
        compiler = self.adapter.get_compiler()
        return compiler.compile_node(self.node, manifest, {})


class CompileTask(GraphRunnableTask):
    def raise_on_first_error(self):
        return True

    def get_node_selector(self) -> ResourceTypeSelector:
        if self.manifest is None or self.graph is None:
            raise DbtInternalError("manifest and graph must be set to get perform node selection")
        return ResourceTypeSelector(
            graph=self.graph,
            manifest=self.manifest,
            previous_state=self.previous_state,
            resource_types=NodeType.executable(),
        )

    def get_runner_type(self, _):
        return CompileRunner

    def task_end_messages(self, results):
        fire_event(CompileComplete())

    def _get_deferred_manifest(self) -> Optional[WritableManifest]:
        if not self.args.defer:
            return None

        state = self.previous_state
        if state is None:
            raise DbtRuntimeError(
                "Received a --defer argument, but no value was provided to --state"
            )

        if state.manifest is None:
            raise DbtRuntimeError(f'Could not find manifest in --state path: "{self.args.state}"')
        return state.manifest

    def defer_to_manifest(self, adapter, selected_uids: AbstractSet[str]):
        deferred_manifest = self._get_deferred_manifest()
        if deferred_manifest is None:
            return
        if self.manifest is None:
            raise DbtInternalError(
                "Expected to defer to manifest, but there is no runtime manifest to defer from!"
            )
        self.manifest.merge_from_artifact(
            adapter=adapter,
            other=deferred_manifest,
            selected=selected_uids,
            favor_state=bool(self.args.favor_state),
        )
        # TODO: is it wrong to write the manifest here? I think it's right...
        self.write_manifest()



import dataclasses
from datetime import datetime
from typing import List, Tuple, ClassVar, Type, TypeVar, Dict, Any, Optional

from dbt.clients.system import write_json, read_json
from dbt import deprecations
from dbt.exceptions import (
    DbtInternalError,
    DbtRuntimeError,
    IncompatibleSchemaError,
)
from dbt.version import __version__
from dbt.events.functions import get_invocation_id, get_metadata_vars
from dbt.dataclass_schema import dbtClassMixin

from dbt.dataclass_schema import (
    ValidatedStringMixin,
    ValidationError,
    register_pattern,
)


SourceKey = Tuple[str, str]


def list_str() -> List[str]:
    """Mypy gets upset about stuff like:

    from dataclasses import dataclass, field
    from typing import Optional, List

    @dataclass
    class Foo:
        x: Optional[List[str]] = field(default_factory=list)


    Because `list` could be any kind of list, I guess
    """
    return []


class Replaceable:
    def replace(self, **kwargs):
        return dataclasses.replace(self, **kwargs)


class Mergeable(Replaceable):
    def merged(self, *args):
        """Perform a shallow merge, where the last non-None write wins. This is
        intended to merge dataclasses that are a collection of optional values.
        """
        replacements = {}
        cls = type(self)
        for arg in args:
            for field in dataclasses.fields(cls):
                value = getattr(arg, field.name)
                if value is not None:
                    replacements[field.name] = value

        return self.replace(**replacements)


class Writable:
    def write(self, path: str):
        write_json(path, self.to_dict(omit_none=False))  # type: ignore


class AdditionalPropertiesMixin:
    """Make this class an extensible property.

    The underlying class definition must include a type definition for a field
    named '_extra' that is of type `Dict[str, Any]`.
    """

    ADDITIONAL_PROPERTIES = True

    # This takes attributes in the dictionary that are
    # not in the class definitions and puts them in an
    # _extra dict in the class
    @classmethod
    def __pre_deserialize__(cls, data):
        # dir() did not work because fields with
        # metadata settings are not found
        # The original version of this would create the
        # object first and then update extra with the
        # extra keys, but that won't work here, so
        # we're copying the dict so we don't insert the
        # _extra in the original data. This also requires
        # that Mashumaro actually build the '_extra' field
        cls_keys = cls._get_field_names()
        new_dict = {}
        for key, value in data.items():
            if key not in cls_keys and key != "_extra":
                if "_extra" not in new_dict:
                    new_dict["_extra"] = {}
                new_dict["_extra"][key] = value
            else:
                new_dict[key] = value
        data = new_dict
        data = super().__pre_deserialize__(data)
        return data

    def __post_serialize__(self, dct):
        data = super().__post_serialize__(dct)
        data.update(self.extra)
        if "_extra" in data:
            del data["_extra"]
        return data

    def replace(self, **kwargs):
        dct = self.to_dict(omit_none=False)
        dct.update(kwargs)
        return self.from_dict(dct)

    @property
    def extra(self):
        return self._extra


class Readable:
    @classmethod
    def read(cls, path: str):
        try:
            data = read_json(path)
        except (EnvironmentError, ValueError) as exc:
            raise DbtRuntimeError(
                f'Could not read {cls.__name__} at "{path}" as JSON: {exc}'
            ) from exc

        return cls.from_dict(data)  # type: ignore


BASE_SCHEMAS_URL = "https://schemas.getdbt.com/"
SCHEMA_PATH = "dbt/{name}/v{version}.json"


@dataclasses.dataclass
class SchemaVersion:
    name: str
    version: int

    @property
    def path(self) -> str:
        return SCHEMA_PATH.format(name=self.name, version=self.version)

    def __str__(self) -> str:
        return BASE_SCHEMAS_URL + self.path


# This is used in the ManifestMetadata, RunResultsMetadata, RunOperationResultMetadata,
# FreshnessMetadata, and CatalogMetadata classes
@dataclasses.dataclass
class BaseArtifactMetadata(dbtClassMixin):
    dbt_schema_version: str
    dbt_version: str = __version__
    generated_at: datetime = dataclasses.field(default_factory=datetime.utcnow)
    invocation_id: Optional[str] = dataclasses.field(default_factory=get_invocation_id)
    env: Dict[str, str] = dataclasses.field(default_factory=get_metadata_vars)

    def __post_serialize__(self, dct):
        dct = super().__post_serialize__(dct)
        if dct["generated_at"] and dct["generated_at"].endswith("+00:00"):
            dct["generated_at"] = dct["generated_at"].replace("+00:00", "") + "Z"
        return dct


# This is used as a class decorator to set the schema_version in the
# 'dbt_schema_version' class attribute. (It's copied into the metadata objects.)
# Name attributes of SchemaVersion in classes with the 'schema_version' decorator:
#   manifest
#   run-results
#   run-operation-result
#   sources
#   catalog
#   remote-compile-result
#   remote-execution-result
#   remote-run-result
def schema_version(name: str, version: int):
    def inner(cls: Type[VersionedSchema]):
        cls.dbt_schema_version = SchemaVersion(
            name=name,
            version=version,
        )
        return cls

    return inner


def get_manifest_schema_version(dct: dict) -> int:
    schema_version = dct.get("metadata", {}).get("dbt_schema_version", None)
    if not schema_version:
        raise ValueError("Manifest doesn't have schema version")
    return int(schema_version.split(".")[-2][-1])


# we renamed these properties in v1.3
# this method allows us to be nice to the early adopters
def rename_metric_attr(data: dict, raise_deprecation_warning: bool = False) -> dict:
    metric_name = data["name"]
    if raise_deprecation_warning and (
        "sql" in data.keys()
        or "type" in data.keys()
        or data.get("calculation_method") == "expression"
    ):
        deprecations.warn("metric-attr-renamed", metric_name=metric_name)
    duplicated_attribute_msg = """\n
The metric '{}' contains both the deprecated metric property '{}'
and the up-to-date metric property '{}'. Please remove the deprecated property.
"""
    if "sql" in data.keys():
        if "expression" in data.keys():
            raise ValidationError(
                duplicated_attribute_msg.format(metric_name, "sql", "expression")
            )
        else:
            data["expression"] = data.pop("sql")
    if "type" in data.keys():
        if "calculation_method" in data.keys():
            raise ValidationError(
                duplicated_attribute_msg.format(metric_name, "type", "calculation_method")
            )
        else:
            calculation_method = data.pop("type")
            data["calculation_method"] = calculation_method
    # we also changed "type: expression" -> "calculation_method: derived"
    if data.get("calculation_method") == "expression":
        data["calculation_method"] = "derived"
    return data


def rename_sql_attr(node_content: dict) -> dict:
    if "raw_sql" in node_content:
        node_content["raw_code"] = node_content.pop("raw_sql")
    if "compiled_sql" in node_content:
        node_content["compiled_code"] = node_content.pop("compiled_sql")
    node_content["language"] = "sql"
    return node_content


def upgrade_node_content(node_content):
    rename_sql_attr(node_content)
    if node_content["resource_type"] != "seed" and "root_path" in node_content:
        del node_content["root_path"]


def upgrade_seed_content(node_content):
    # Remove compilation related attributes
    for attr_name in (
        "language",
        "refs",
        "sources",
        "metrics",
        "depends_on",
        "compiled_path",
        "compiled",
        "compiled_code",
        "extra_ctes_injected",
        "extra_ctes",
        "relation_name",
    ):
        if attr_name in node_content:
            del node_content[attr_name]


def upgrade_manifest_json(manifest: dict) -> dict:
    for node_content in manifest.get("nodes", {}).values():
        upgrade_node_content(node_content)
        if node_content["resource_type"] == "seed":
            upgrade_seed_content(node_content)
    for disabled in manifest.get("disabled", {}).values():
        # There can be multiple disabled nodes for the same unique_id
        # so make sure all the nodes get the attr renamed
        for node_content in disabled:
            upgrade_node_content(node_content)
            if node_content["resource_type"] == "seed":
                upgrade_seed_content(node_content)
    for metric_content in manifest.get("metrics", {}).values():
        # handle attr renames + value translation ("expression" -> "derived")
        metric_content = rename_metric_attr(metric_content)
        if "root_path" in metric_content:
            del metric_content["root_path"]
    for exposure_content in manifest.get("exposures", {}).values():
        if "root_path" in exposure_content:
            del exposure_content["root_path"]
    for source_content in manifest.get("sources", {}).values():
        if "root_path" in exposure_content:
            del source_content["root_path"]
    for macro_content in manifest.get("macros", {}).values():
        if "root_path" in macro_content:
            del macro_content["root_path"]
    for doc_content in manifest.get("docs", {}).values():
        if "root_path" in doc_content:
            del doc_content["root_path"]
        doc_content["resource_type"] = "doc"
    return manifest


# This is used in the ArtifactMixin and RemoteResult classes
@dataclasses.dataclass
class VersionedSchema(dbtClassMixin):
    dbt_schema_version: ClassVar[SchemaVersion]

    @classmethod
    def json_schema(cls, embeddable: bool = False) -> Dict[str, Any]:
        result = super().json_schema(embeddable=embeddable)
        if not embeddable:
            result["$id"] = str(cls.dbt_schema_version)
        return result

    @classmethod
    def is_compatible_version(cls, schema_version):
        compatible_versions = [str(cls.dbt_schema_version)]
        if hasattr(cls, "compatible_previous_versions"):
            for name, version in cls.compatible_previous_versions():
                compatible_versions.append(str(SchemaVersion(name, version)))
        return str(schema_version) in compatible_versions

    @classmethod
    def read_and_check_versions(cls, path: str):
        try:
            data = read_json(path)
        except (EnvironmentError, ValueError) as exc:
            raise DbtRuntimeError(
                f'Could not read {cls.__name__} at "{path}" as JSON: {exc}'
            ) from exc

        # Check metadata version. There is a class variable 'dbt_schema_version', but
        # that doesn't show up in artifacts, where it only exists in the 'metadata'
        # dictionary.
        if hasattr(cls, "dbt_schema_version"):
            if "metadata" in data and "dbt_schema_version" in data["metadata"]:
                previous_schema_version = data["metadata"]["dbt_schema_version"]
                # cls.dbt_schema_version is a SchemaVersion object
                if not cls.is_compatible_version(previous_schema_version):
                    raise IncompatibleSchemaError(
                        expected=str(cls.dbt_schema_version),
                        found=previous_schema_version,
                    )
        if get_manifest_schema_version(data) <= 7:
            data = upgrade_manifest_json(data)
        return cls.from_dict(data)  # type: ignore


T = TypeVar("T", bound="ArtifactMixin")


# metadata should really be a Generic[T_M] where T_M is a TypeVar bound to
# BaseArtifactMetadata. Unfortunately this isn't possible due to a mypy issue:
# https://github.com/python/mypy/issues/7520
# This is used in the WritableManifest, RunResultsArtifact, RunOperationResultsArtifact,
# and CatalogArtifact
@dataclasses.dataclass(init=False)
class ArtifactMixin(VersionedSchema, Writable, Readable):
    metadata: BaseArtifactMetadata

    @classmethod
    def validate(cls, data):
        super().validate(data)
        if cls.dbt_schema_version is None:
            raise DbtInternalError("Cannot call from_dict with no schema version!")


class Identifier(ValidatedStringMixin):
    ValidationRegex = r"^[^\d\W]\w*$"

    @classmethod
    def is_valid(cls, value: Any) -> bool:
        if not isinstance(value, str):
            return False

        try:
            cls.validate(value)
        except ValidationError:
            return False

        return True


register_pattern(Identifier, r"^[^\d\W]\w*$")



from dataclasses import dataclass
from dbt.events.types import InfoLevel, DebugLevel, WarnLevel, ErrorLevel
from dbt.events.base_types import NoFile
from dbt.events import proto_types as pl
from dbt.events.proto_types import EventInfo  # noqa


# Keeping log messages for testing separate since they are used for debugging.
# Reuse the existing messages when adding logs to tests.


@dataclass
class IntegrationTestInfo(InfoLevel, NoFile, pl.IntegrationTestInfo):
    def code(self):
        return "T001"

    def message(self) -> str:
        return f"Integration Test: {self.msg}"


@dataclass
class IntegrationTestDebug(DebugLevel, NoFile, pl.IntegrationTestDebug):
    def code(self):
        return "T002"

    def message(self) -> str:
        return f"Integration Test: {self.msg}"


@dataclass
class IntegrationTestWarn(WarnLevel, NoFile, pl.IntegrationTestWarn):
    def code(self):
        return "T003"

    def message(self) -> str:
        return f"Integration Test: {self.msg}"


@dataclass
class IntegrationTestError(ErrorLevel, NoFile, pl.IntegrationTestError):
    def code(self):
        return "T004"

    def message(self) -> str:
        return f"Integration Test: {self.msg}"


@dataclass
class IntegrationTestException(ErrorLevel, NoFile, pl.IntegrationTestException):
    def code(self):
        return "T005"

    def message(self) -> str:
        return f"Integration Test: {self.msg}"


@dataclass
class UnitTestInfo(InfoLevel, NoFile, pl.UnitTestInfo):
    def code(self):
        return "T006"

    def message(self) -> str:
        return f"Unit Test: {self.msg}"



# TODO  Move this to /core/dbt/flags.py when we're ready to break things
import os
from dataclasses import dataclass
from multiprocessing import get_context
from pprint import pformat as pf

from click import get_current_context

if os.name != "nt":
    # https://bugs.python.org/issue41567
    import multiprocessing.popen_spawn_posix  # type: ignore  # noqa: F401


@dataclass(frozen=True)
class Flags:
    def __init__(self, ctx=None) -> None:

        if ctx is None:
            ctx = get_current_context()

        def assign_params(ctx):
            """Recursively adds all click params to flag object"""
            for param_name, param_value in ctx.params.items():
                # N.B. You have to use the base MRO method (object.__setattr__) to set attributes
                # when using frozen dataclasses.
                # https://docs.python.org/3/library/dataclasses.html#frozen-instances
                if hasattr(self, param_name):
                    raise Exception(f"Duplicate flag names found in click command: {param_name}")
                object.__setattr__(self, param_name.upper(), param_value)
            if ctx.parent:
                assign_params(ctx.parent)

        assign_params(ctx)

        # Hard coded flags
        object.__setattr__(self, "WHICH", ctx.info_name)
        object.__setattr__(self, "MP_CONTEXT", get_context("spawn"))

        # Support console DO NOT TRACK initiave
        if os.getenv("DO_NOT_TRACK", "").lower() in (1, "t", "true", "y", "yes"):
            object.__setattr__(self, "ANONYMOUS_USAGE_STATS", False)

    def __str__(self) -> str:
        return str(pf(self.__dict__))



import os
from typing import List
from dbt.constants import SECRET_ENV_PREFIX


def env_secrets() -> List[str]:
    return [v for k, v in os.environ.items() if k.startswith(SECRET_ENV_PREFIX) and v.strip()]


def scrub_secrets(msg: str, secrets: List[str]) -> str:
    scrubbed = msg

    for secret in secrets:
        scrubbed = scrubbed.replace(secret, "*****")

    return scrubbed



import os
from dbt.config.project import Project
from dbt.config.renderer import DbtProjectYamlRenderer
from dbt.contracts.results import RunningStatus, collect_timing_info
from dbt.events.functions import fire_event
from dbt.events.types import NodeCompiling, NodeExecuting
from dbt.exceptions import DbtRuntimeError
from dbt import flags
from dbt.task.sql import SqlCompileRunner
from dataclasses import dataclass


@dataclass
class RuntimeArgs:
    project_dir: str
    profiles_dir: str
    single_threaded: bool
    profile: str
    target: str


class SqlCompileRunnerNoIntrospection(SqlCompileRunner):
    def compile_and_execute(self, manifest, ctx):
        """
        This version of this method does not connect to the data warehouse.
        As a result, introspective queries at compilation will not be supported
        and will throw an error.

        TODO: This is a temporary solution to more complex permissions requirements
        for the semantic layer, and thus largely duplicates the code in the parent class
        method. Once conditional credential usage is enabled, this should be removed.
        """
        result = None
        ctx.node.update_event_status(node_status=RunningStatus.Compiling)
        fire_event(
            NodeCompiling(
                node_info=ctx.node.node_info,
            )
        )
        with collect_timing_info("compile") as timing_info:
            # if we fail here, we still have a compiled node to return
            # this has the benefit of showing a build path for the errant
            # model
            ctx.node = self.compile(manifest)
        ctx.timing.append(timing_info)

        # for ephemeral nodes, we only want to compile, not run
        if not ctx.node.is_ephemeral_model:
            ctx.node.update_event_status(node_status=RunningStatus.Executing)
            fire_event(
                NodeExecuting(
                    node_info=ctx.node.node_info,
                )
            )
            with collect_timing_info("execute") as timing_info:
                result = self.run(ctx.node, manifest)
                ctx.node = result.node

            ctx.timing.append(timing_info)

        return result


def get_dbt_config(project_dir, args=None, single_threaded=False):
    from dbt.config.runtime import RuntimeConfig
    import dbt.adapters.factory
    import dbt.events.functions

    if os.getenv("DBT_PROFILES_DIR"):
        profiles_dir = os.getenv("DBT_PROFILES_DIR")
    else:
        profiles_dir = flags.DEFAULT_PROFILES_DIR

    profile_name = getattr(args, "profile", None)

    runtime_args = RuntimeArgs(
        project_dir=project_dir,
        profiles_dir=profiles_dir,
        single_threaded=single_threaded,
        profile=profile_name,
        target=getattr(args, "target", None),
    )

    profile = RuntimeConfig.collect_profile(args=runtime_args, profile_name=profile_name)
    project_renderer = DbtProjectYamlRenderer(profile, None)
    project = RuntimeConfig.collect_project(args=runtime_args, project_renderer=project_renderer)
    assert type(project) is Project

    config = RuntimeConfig.from_parts(project, profile, runtime_args)

    # Set global flags from arguments
    flags.set_from_args(args, config)

    # This is idempotent, so we can call it repeatedly
    dbt.adapters.factory.register_adapter(config)

    # Make sure we have a valid invocation_id
    dbt.events.functions.set_invocation_id()
    dbt.events.functions.reset_metadata_vars()

    return config


def get_task_by_type(type):
    from dbt.task.run import RunTask
    from dbt.task.list import ListTask
    from dbt.task.seed import SeedTask
    from dbt.task.test import TestTask
    from dbt.task.build import BuildTask
    from dbt.task.snapshot import SnapshotTask
    from dbt.task.run_operation import RunOperationTask

    if type == "run":
        return RunTask
    elif type == "test":
        return TestTask
    elif type == "list":
        return ListTask
    elif type == "seed":
        return SeedTask
    elif type == "build":
        return BuildTask
    elif type == "snapshot":
        return SnapshotTask
    elif type == "run_operation":
        return RunOperationTask

    raise DbtRuntimeError("not a valid task")


def create_task(type, args, manifest, config):
    task = get_task_by_type(type)

    def no_op(*args, **kwargs):
        pass

    task = task(args, config)
    task.load_manifest = no_op
    task.manifest = manifest
    return task


def _get_operation_node(manifest, project_path, sql, node_name):
    from dbt.parser.manifest import process_node
    from dbt.parser.sql import SqlBlockParser
    import dbt.adapters.factory

    config = get_dbt_config(project_path)
    block_parser = SqlBlockParser(
        project=config,
        manifest=manifest,
        root_project=config,
    )

    adapter = dbt.adapters.factory.get_adapter(config)
    sql_node = block_parser.parse_remote(sql, node_name)
    process_node(config, manifest, sql_node)
    return config, sql_node, adapter


def compile_sql(manifest, project_path, sql, node_name="query"):
    config, node, adapter = _get_operation_node(manifest, project_path, sql, node_name)
    allow_introspection = str(os.environ.get("__DBT_ALLOW_INTROSPECTION", "1")).lower() in (
        "true",
        "1",
        "on",
    )

    if allow_introspection:
        runner = SqlCompileRunner(config, adapter, node, 1, 1)
    else:
        runner = SqlCompileRunnerNoIntrospection(config, adapter, node, 1, 1)
    return runner.safe_run(manifest)


def execute_sql(manifest, project_path, sql, node_name="query"):
    from dbt.task.sql import SqlExecuteRunner

    config, node, adapter = _get_operation_node(manifest, project_path, sql, node_name)

    runner = SqlExecuteRunner(config, adapter, node, 1, 1)

    return runner.safe_run(manifest)


def parse_to_manifest(config):
    from dbt.parser.manifest import ManifestLoader

    return ManifestLoader.get_full_manifest(config)


def deserialize_manifest(manifest_msgpack):
    from dbt.contracts.graph.manifest import Manifest

    return Manifest.from_msgpack(manifest_msgpack)


def serialize_manifest(manifest):
    return manifest.to_msgpack()



from dataclasses import dataclass
from enum import Enum
import os
import threading
from datetime import datetime
import dbt.events.proto_types as pt
import sys

if sys.version_info >= (3, 8):
    from typing import Protocol
else:
    from typing_extensions import Protocol

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# These base types define the _required structure_ for the concrete event #
# types defined in types.py                                               #
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #


class Cache:
    # Events with this class will only be logged when the `--log-cache-events` flag is passed
    pass


def get_global_metadata_vars() -> dict:
    from dbt.events.functions import get_metadata_vars

    return get_metadata_vars()


def get_invocation_id() -> str:
    from dbt.events.functions import get_invocation_id

    return get_invocation_id()


# exactly one pid per concrete event
def get_pid() -> int:
    return os.getpid()


# preformatted time stamp
def get_ts_rfc3339() -> str:
    ts = datetime.utcnow()
    ts_rfc3339 = ts.strftime("%Y-%m-%dT%H:%M:%S.%fZ")
    return ts_rfc3339


# in theory threads can change so we don't cache them.
def get_thread_name() -> str:
    return threading.current_thread().name


# EventLevel is an Enum, but mixing in the 'str' type is suggested in the Python
# documentation, and provides support for json conversion, which fails otherwise.
class EventLevel(str, Enum):
    DEBUG = "debug"
    TEST = "test"
    INFO = "info"
    WARN = "warn"
    ERROR = "error"


@dataclass
class BaseEvent:
    """BaseEvent for proto message generated python events"""

    #   def __post_init__(self):
    #       super().__post_init__()
    #       if not self.info.level:
    #           self.info.level = self.level_tag()
    #       assert self.info.level in ["info", "warn", "error", "debug", "test"]
    #       if not hasattr(self.info, "msg") or not self.info.msg:
    #           self.info.msg = self.message()
    #       self.info.invocation_id = get_invocation_id()
    #       self.info.extra = get_global_metadata_vars()
    #       self.info.ts = datetime.utcnow()
    #       self.info.pid = get_pid()
    #       self.info.thread = get_thread_name()
    #       self.info.code = self.code()
    #       self.info.name = type(self).__name__

    def level_tag(self) -> EventLevel:
        return EventLevel.DEBUG

    def message(self) -> str:
        raise Exception("message() not implemented for event")

    def code(self) -> str:
        raise Exception("code() not implemented for event")


class EventMsg(Protocol):
    info: pt.EventInfo
    data: BaseEvent


def msg_from_base_event(event: BaseEvent, level: EventLevel = None):

    msg_class_name = f"{type(event).__name__}Msg"
    msg_cls = getattr(pt, msg_class_name)

    # level in EventInfo must be a string, not an EventLevel
    msg_level: str = level.value if level else event.level_tag().value
    assert msg_level is not None
    event_info = pt.EventInfo(
        level=msg_level,
        msg=event.message(),
        invocation_id=get_invocation_id(),
        extra=get_global_metadata_vars(),
        ts=datetime.utcnow(),
        pid=get_pid(),
        thread=get_thread_name(),
        code=event.code(),
        name=type(event).__name__,
    )
    new_event = msg_cls(data=event, info=event_info)
    return new_event


# DynamicLevel requires that the level be supplied on the
# event construction call using the "info" function from functions.py
@dataclass  # type: ignore[misc]
class DynamicLevel(BaseEvent):
    pass


@dataclass
class TestLevel(BaseEvent):
    __test__ = False

    def level_tag(self) -> EventLevel:
        return EventLevel.TEST


@dataclass  # type: ignore[misc]
class DebugLevel(BaseEvent):
    def level_tag(self) -> EventLevel:
        return EventLevel.DEBUG


@dataclass  # type: ignore[misc]
class InfoLevel(BaseEvent):
    def level_tag(self) -> EventLevel:
        return EventLevel.INFO


@dataclass  # type: ignore[misc]
class WarnLevel(BaseEvent):
    def level_tag(self) -> EventLevel:
        return EventLevel.WARN


@dataclass  # type: ignore[misc]
class ErrorLevel(BaseEvent):
    def level_tag(self) -> EventLevel:
        return EventLevel.ERROR


# Included to ensure classes with str-type message members are initialized correctly.
@dataclass  # type: ignore[misc]
class AdapterEventStringFunctor:
    def __post_init__(self):
        super().__post_init__()
        if not isinstance(self.base_msg, str):
            self.base_msg = str(self.base_msg)


@dataclass  # type: ignore[misc]
class EventStringFunctor:
    def __post_init__(self):
        super().__post_init__()
        if not isinstance(self.msg, str):
            self.msg = str(self.msg)


# prevents an event from going to the file
# This should rarely be used in core code. It is currently
# only used in integration tests and for the 'clean' command.
class NoFile:
    pass


# prevents an event from going to stdout
class NoStdOut:
    pass



from pathlib import Path, PurePath

import click
from dbt.cli.option_types import YAML, WarnErrorOptionsType
from dbt.cli.resolvers import default_project_dir, default_profiles_dir


# TODO:  The name (reflected in flags) is a correction!
# The original name was `SEND_ANONYMOUS_USAGE_STATS` and used an env var called "DBT_SEND_ANONYMOUS_USAGE_STATS"
# Both of which break existing naming conventions (doesn't match param flag).
# This will need to be fixed before use in the main codebase and communicated as a change to the community!
anonymous_usage_stats = click.option(
    "--anonymous-usage-stats/--no-anonymous-usage-stats",
    envvar="DBT_ANONYMOUS_USAGE_STATS",
    help="Send anonymous usage stats to dbt Labs.",
    default=True,
)

args = click.option(
    "--args",
    envvar=None,
    help="Supply arguments to the macro. This dictionary will be mapped to the keyword arguments defined in the selected macro. This argument should be a YAML string, eg. '{my_variable: my_value}'",
    type=YAML(),
)

browser = click.option(
    "--browser/--no-browser",
    envvar=None,
    help="Wether or not to open a local web browser after starting the server",
    default=True,
)

cache_selected_only = click.option(
    "--cache-selected-only/--no-cache-selected-only",
    envvar="DBT_CACHE_SELECTED_ONLY",
    help="Pre cache database objects relevant to selected resource only.",
)

compile_docs = click.option(
    "--compile/--no-compile",
    envvar=None,
    help="Wether or not to run 'dbt compile' as part of docs generation",
    default=True,
)

compile_parse = click.option(
    "--compile/--no-compile",
    envvar=None,
    help="TODO: No help text currently available",
    default=True,
)

config_dir = click.option(
    "--config-dir",
    envvar=None,
    help="If specified, DBT will show path information for this project",
    type=click.STRING,
)

debug = click.option(
    "--debug/--no-debug",
    "-d/ ",
    envvar="DBT_DEBUG",
    help="Display debug logging during dbt execution. Useful for debugging and making bug reports.",
)

# TODO:  The env var and name (reflected in flags) are corrections!
# The original name was `DEFER_MODE` and used an env var called "DBT_DEFER_TO_STATE"
# Both of which break existing naming conventions.
# This will need to be fixed before use in the main codebase and communicated as a change to the community!
defer = click.option(
    "--defer/--no-defer",
    envvar="DBT_DEFER",
    help="If set, defer to the state variable for resolving unselected nodes.",
)

enable_legacy_logger = click.option(
    "--enable-legacy-logger/--no-enable-legacy-logger",
    envvar="DBT_ENABLE_LEGACY_LOGGER",
    hidden=True,
)

exclude = click.option("--exclude", envvar=None, help="Specify the nodes to exclude.")

fail_fast = click.option(
    "--fail-fast/--no-fail-fast",
    "-x/ ",
    envvar="DBT_FAIL_FAST",
    help="Stop execution on first failure.",
)

full_refresh = click.option(
    "--full-refresh",
    "-f",
    envvar="DBT_FULL_REFRESH",
    help="If specified, dbt will drop incremental models and fully-recalculate the incremental table from the model definition.",
    is_flag=True,
)

indirect_selection = click.option(
    "--indirect-selection",
    envvar="DBT_INDIRECT_SELECTION",
    help="Select all tests that are adjacent to selected resources, even if they those resources have been explicitly selected.",
    type=click.Choice(["eager", "cautious"], case_sensitive=False),
    default="eager",
)

log_cache_events = click.option(
    "--log-cache-events/--no-log-cache-events",
    help="Enable verbose adapter cache logging.",
    envvar="DBT_LOG_CACHE_EVENTS",
)

log_format = click.option(
    "--log-format",
    envvar="DBT_LOG_FORMAT",
    help="Specify the log format, overriding the command's default.",
    type=click.Choice(["text", "json", "default"], case_sensitive=False),
    default="default",
)

log_path = click.option(
    "--log-path",
    envvar="DBT_LOG_PATH",
    help="Configure the 'log-path'. Only applies this setting for the current run. Overrides the 'DBT_LOG_PATH' if it is set.",
    type=click.Path(),
)

macro_debugging = click.option(
    "--macro-debugging/--no-macro-debugging",
    envvar="DBT_MACRO_DEBUGGING",
    hidden=True,
)

models = click.option(
    "-m",
    "-s",
    "models",
    envvar=None,
    help="Specify the nodes to include.",
    multiple=True,
)

output = click.option(
    "--output",
    envvar=None,
    help="TODO: No current help text",
    type=click.Choice(["json", "name", "path", "selector"], case_sensitive=False),
    default="name",
)

output_keys = click.option(
    "--output-keys", envvar=None, help="TODO: No current help text", type=click.STRING
)

output_path = click.option(
    "--output",
    "-o",
    envvar=None,
    help="Specify the output path for the json report. By default, outputs to 'target/sources.json'",
    type=click.Path(file_okay=True, dir_okay=False, writable=True),
    default=PurePath.joinpath(Path.cwd(), "target/sources.json"),
)

parse_only = click.option(
    "--parse-only",
    envvar=None,
    help="TODO:  No help text currently available",
    is_flag=True,
)

partial_parse = click.option(
    "--partial-parse/--no-partial-parse",
    envvar="DBT_PARTIAL_PARSE",
    help="Allow for partial parsing by looking for and writing to a pickle file in the target directory. This overrides the user configuration file.",
    default=True,
)

port = click.option(
    "--port",
    envvar=None,
    help="Specify the port number for the docs server",
    default=8080,
    type=click.INT,
)

# TODO:  The env var and name (reflected in flags) are corrections!
# The original name was `NO_PRINT` and used the env var `DBT_NO_PRINT`.
# Both of which break existing naming conventions.
# This will need to be fixed before use in the main codebase and communicated as a change to the community!
print = click.option(
    "--print/--no-print",
    envvar="DBT_PRINT",
    help="Output all {{ print() }} macro calls.",
    default=True,
)

printer_width = click.option(
    "--printer-width",
    envvar="DBT_PRINTER_WIDTH",
    help="Sets the width of terminal output",
    type=click.INT,
    default=80,
)

profile = click.option(
    "--profile",
    envvar=None,
    help="Which profile to load. Overrides setting in dbt_project.yml.",
)

profiles_dir = click.option(
    "--profiles-dir",
    envvar="DBT_PROFILES_DIR",
    help="Which directory to look in for the profiles.yml file. If not set, dbt will look in the current working directory first, then HOME/.dbt/",
    default=default_profiles_dir(),
    type=click.Path(exists=True),
)

project_dir = click.option(
    "--project-dir",
    envvar=None,
    help="Which directory to look in for the dbt_project.yml file. Default is the current working directory and its parents.",
    default=default_project_dir(),
    type=click.Path(exists=True),
)

quiet = click.option(
    "--quiet/--no-quiet",
    envvar="DBT_QUIET",
    help="Suppress all non-error logging to stdout. Does not affect {{ print() }} macro calls.",
)

record_timing_info = click.option(
    "--record-timing-info",
    "-r",
    envvar=None,
    help="When this option is passed, dbt will output low-level timing stats to the specified file. Example: `--record-timing-info output.profile`",
    type=click.Path(exists=False),
)

resource_type = click.option(
    "--resource-type",
    envvar=None,
    help="TODO: No current help text",
    type=click.Choice(
        [
            "metric",
            "source",
            "analysis",
            "model",
            "test",
            "exposure",
            "snapshot",
            "seed",
            "default",
            "all",
        ],
        case_sensitive=False,
    ),
    default="default",
)

selector = click.option(
    "--selector", envvar=None, help="The selector name to use, as defined in selectors.yml"
)

show = click.option(
    "--show", envvar=None, help="Show a sample of the loaded data in the terminal", is_flag=True
)

skip_profile_setup = click.option(
    "--skip-profile-setup", "-s", envvar=None, help="Skip interactive profile setup.", is_flag=True
)

# TODO:  The env var and name (reflected in flags) are corrections!
# The original name was `ARTIFACT_STATE_PATH` and used the env var `DBT_ARTIFACT_STATE_PATH`.
# Both of which break existing naming conventions.
# This will need to be fixed before use in the main codebase and communicated as a change to the community!
state = click.option(
    "--state",
    envvar="DBT_STATE",
    help="If set, use the given directory as the source for json files to compare with this project.",
    type=click.Path(
        dir_okay=True,
        exists=True,
        file_okay=False,
        readable=True,
        resolve_path=True,
    ),
)

static_parser = click.option(
    "--static-parser/--no-static-parser",
    envvar="DBT_STATIC_PARSER",
    help="Use the static parser.",
    default=True,
)

store_failures = click.option(
    "--store-failures",
    envvar="DBT_STORE_FAILURES",
    help="Store test results (failing rows) in the database",
    is_flag=True,
)

target = click.option(
    "--target", "-t", envvar=None, help="Which target to load for the given profile"
)

target_path = click.option(
    "--target-path",
    envvar="DBT_TARGET_PATH",
    help="Configure the 'target-path'. Only applies this setting for the current run. Overrides the 'DBT_TARGET_PATH' if it is set.",
    type=click.Path(),
)

threads = click.option(
    "--threads",
    envvar=None,
    help="Specify number of threads to use while executing models. Overrides settings in profiles.yml.",
    default=1,
    type=click.INT,
)

use_colors = click.option(
    "--use-colors/--no-use-colors",
    envvar="DBT_USE_COLORS",
    help="Output is colorized by default and may also be set in a profile or at the command line.",
    default=True,
)

use_experimental_parser = click.option(
    "--use-experimental-parser/--no-use-experimental-parser",
    envvar="DBT_USE_EXPERIMENTAL_PARSER",
    help="Enable experimental parsing features.",
)

vars = click.option(
    "--vars",
    envvar=None,
    help="Supply variables to the project. This argument overrides variables defined in your dbt_project.yml file. This argument should be a YAML string, eg. '{my_variable: my_value}'",
    type=YAML(),
)

version = click.option(
    "--version",
    envvar=None,
    help="Show version information",
    is_flag=True,
)

version_check = click.option(
    "--version-check/--no-version-check",
    envvar="DBT_VERSION_CHECK",
    help="Ensure dbt's version matches the one specified in the dbt_project.yml file ('require-dbt-version')",
    default=True,
)

warn_error = click.option(
    "--warn-error",
    envvar="DBT_WARN_ERROR",
    help="If dbt would normally warn, instead raise an exception. Examples include --select that selects nothing, deprecations, configurations with no associated models, invalid test configurations, and missing sources/refs in tests.",
    default=None,
    flag_value=True,
)

warn_error_options = click.option(
    "--warn-error-options",
    envvar="DBT_WARN_ERROR_OPTIONS",
    default=None,
    help="""If dbt would normally warn, instead raise an exception based on include/exclude configuration. Examples include --select that selects nothing, deprecations, configurations with no associated models, invalid test configurations,
    and missing sources/refs in tests. This argument should be a YAML string, with keys 'include' or 'exclude'. eg. '{"include": "all", "exclude": ["NoNodesForSelectionCriteria"]}'""",
    type=WarnErrorOptionsType(),
)

write_json = click.option(
    "--write-json/--no-write-json",
    envvar="DBT_WRITE_JSON",
    help="Writing the manifest and run_results.json files to disk",
    default=True,
)

write_manifest = click.option(
    "--write-manifest/--no-write-manifest",
    envvar=None,
    help="TODO: No help text currently available",
    default=True,
)



from pathlib import Path


def default_project_dir():
    paths = list(Path.cwd().parents)
    paths.insert(0, Path.cwd())
    return next((x for x in paths if (x / "dbt_project.yml").exists()), Path.cwd())


def default_profiles_dir():
    return Path.cwd() if (Path.cwd() / "profiles.yml").exists() else Path.home() / ".dbt"



SECRET_ENV_PREFIX = "DBT_ENV_SECRET_"
DEFAULT_ENV_PLACEHOLDER = "DBT_DEFAULT_PLACEHOLDER"
METADATA_ENV_PREFIX = "DBT_ENV_CUSTOM_ENV_"

MAXIMUM_SEED_SIZE = 1 * 1024 * 1024
MAXIMUM_SEED_SIZE_NAME = "1MB"

PIN_PACKAGE_URL = (
    "https://docs.getdbt.com/docs/package-management#section-specifying-package-versions"
)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: types.proto
# plugin: python-betterproto
from dataclasses import dataclass
from datetime import datetime
from typing import Dict, List

import betterproto


@dataclass
class EventInfo(betterproto.Message):
    """Common event info"""

    name: str = betterproto.string_field(1)
    code: str = betterproto.string_field(2)
    msg: str = betterproto.string_field(3)
    level: str = betterproto.string_field(4)
    invocation_id: str = betterproto.string_field(5)
    pid: int = betterproto.int32_field(6)
    thread: str = betterproto.string_field(7)
    ts: datetime = betterproto.message_field(8)
    extra: Dict[str, str] = betterproto.map_field(
        9, betterproto.TYPE_STRING, betterproto.TYPE_STRING
    )
    category: str = betterproto.string_field(10)


@dataclass
class TimingInfoMsg(betterproto.Message):
    """TimingInfo"""

    name: str = betterproto.string_field(1)
    started_at: datetime = betterproto.message_field(2)
    completed_at: datetime = betterproto.message_field(3)


@dataclass
class NodeInfo(betterproto.Message):
    """NodeInfo"""

    node_path: str = betterproto.string_field(1)
    node_name: str = betterproto.string_field(2)
    unique_id: str = betterproto.string_field(3)
    resource_type: str = betterproto.string_field(4)
    materialized: str = betterproto.string_field(5)
    node_status: str = betterproto.string_field(6)
    node_started_at: str = betterproto.string_field(7)
    node_finished_at: str = betterproto.string_field(8)
    meta: Dict[str, str] = betterproto.map_field(
        9, betterproto.TYPE_STRING, betterproto.TYPE_STRING
    )


@dataclass
class RunResultMsg(betterproto.Message):
    """RunResult"""

    status: str = betterproto.string_field(1)
    message: str = betterproto.string_field(2)
    timing_info: List["TimingInfoMsg"] = betterproto.message_field(3)
    thread: str = betterproto.string_field(4)
    execution_time: float = betterproto.float_field(5)
    adapter_response: Dict[str, str] = betterproto.map_field(
        6, betterproto.TYPE_STRING, betterproto.TYPE_STRING
    )
    num_failures: int = betterproto.int32_field(7)


@dataclass
class ReferenceKeyMsg(betterproto.Message):
    """ReferenceKey"""

    database: str = betterproto.string_field(1)
    schema: str = betterproto.string_field(2)
    identifier: str = betterproto.string_field(3)


@dataclass
class ListOfStrings(betterproto.Message):
    """ListOfStrings"""

    value: List[str] = betterproto.string_field(1)


@dataclass
class GenericMessage(betterproto.Message):
    """GenericMessage, used for deserializing only"""

    info: "EventInfo" = betterproto.message_field(1)


@dataclass
class MainReportVersion(betterproto.Message):
    """A001"""

    version: str = betterproto.string_field(1)
    log_version: int = betterproto.int32_field(2)


@dataclass
class MainReportVersionMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "MainReportVersion" = betterproto.message_field(2)


@dataclass
class MainReportArgs(betterproto.Message):
    """A002"""

    args: Dict[str, str] = betterproto.map_field(
        1, betterproto.TYPE_STRING, betterproto.TYPE_STRING
    )


@dataclass
class MainReportArgsMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "MainReportArgs" = betterproto.message_field(2)


@dataclass
class MainTrackingUserState(betterproto.Message):
    """A003"""

    user_state: str = betterproto.string_field(1)


@dataclass
class MainTrackingUserStateMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "MainTrackingUserState" = betterproto.message_field(2)


@dataclass
class MergedFromState(betterproto.Message):
    """A004"""

    num_merged: int = betterproto.int32_field(1)
    sample: List[str] = betterproto.string_field(2)


@dataclass
class MergedFromStateMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "MergedFromState" = betterproto.message_field(2)


@dataclass
class MissingProfileTarget(betterproto.Message):
    """A005"""

    profile_name: str = betterproto.string_field(1)
    target_name: str = betterproto.string_field(2)


@dataclass
class MissingProfileTargetMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "MissingProfileTarget" = betterproto.message_field(2)


@dataclass
class InvalidOptionYAML(betterproto.Message):
    """A008"""

    option_name: str = betterproto.string_field(1)


@dataclass
class InvalidOptionYAMLMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "InvalidOptionYAML" = betterproto.message_field(2)


@dataclass
class LogDbtProjectError(betterproto.Message):
    """A009"""

    exc: str = betterproto.string_field(1)


@dataclass
class LogDbtProjectErrorMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "LogDbtProjectError" = betterproto.message_field(2)


@dataclass
class LogDbtProfileError(betterproto.Message):
    """A011"""

    exc: str = betterproto.string_field(1)
    profiles: List[str] = betterproto.string_field(2)


@dataclass
class LogDbtProfileErrorMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "LogDbtProfileError" = betterproto.message_field(2)


@dataclass
class StarterProjectPath(betterproto.Message):
    """A017"""

    dir: str = betterproto.string_field(1)


@dataclass
class StarterProjectPathMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "StarterProjectPath" = betterproto.message_field(2)


@dataclass
class ConfigFolderDirectory(betterproto.Message):
    """A018"""

    dir: str = betterproto.string_field(1)


@dataclass
class ConfigFolderDirectoryMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "ConfigFolderDirectory" = betterproto.message_field(2)


@dataclass
class NoSampleProfileFound(betterproto.Message):
    """A019"""

    adapter: str = betterproto.string_field(1)


@dataclass
class NoSampleProfileFoundMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "NoSampleProfileFound" = betterproto.message_field(2)


@dataclass
class ProfileWrittenWithSample(betterproto.Message):
    """A020"""

    name: str = betterproto.string_field(1)
    path: str = betterproto.string_field(2)


@dataclass
class ProfileWrittenWithSampleMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "ProfileWrittenWithSample" = betterproto.message_field(2)


@dataclass
class ProfileWrittenWithTargetTemplateYAML(betterproto.Message):
    """A021"""

    name: str = betterproto.string_field(1)
    path: str = betterproto.string_field(2)


@dataclass
class ProfileWrittenWithTargetTemplateYAMLMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "ProfileWrittenWithTargetTemplateYAMLMsg" = betterproto.message_field(2)


@dataclass
class ProfileWrittenWithProjectTemplateYAML(betterproto.Message):
    """A022"""

    name: str = betterproto.string_field(1)
    path: str = betterproto.string_field(2)


@dataclass
class ProfileWrittenWithProjectTemplateYAMLMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "ProfileWrittenWithProjectTemplateYAML" = betterproto.message_field(2)


@dataclass
class SettingUpProfile(betterproto.Message):
    """A023"""

    pass


@dataclass
class SettingUpProfileMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "SettingUpProfile" = betterproto.message_field(2)


@dataclass
class InvalidProfileTemplateYAML(betterproto.Message):
    """A024"""

    pass


@dataclass
class InvalidProfileTemplateYAMLMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "InvalidProfileTemplateYAML" = betterproto.message_field(2)


@dataclass
class ProjectNameAlreadyExists(betterproto.Message):
    """A025"""

    name: str = betterproto.string_field(1)


@dataclass
class ProjectNameAlreadyExistsMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "ProjectNameAlreadyExists" = betterproto.message_field(2)


@dataclass
class ProjectCreated(betterproto.Message):
    """A026"""

    project_name: str = betterproto.string_field(1)
    docs_url: str = betterproto.string_field(2)
    slack_url: str = betterproto.string_field(3)


@dataclass
class ProjectCreatedMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "ProjectCreated" = betterproto.message_field(2)


@dataclass
class PackageRedirectDeprecation(betterproto.Message):
    """D001"""

    old_name: str = betterproto.string_field(1)
    new_name: str = betterproto.string_field(2)


@dataclass
class PackageRedirectDeprecationMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "PackageRedirectDeprecation" = betterproto.message_field(2)


@dataclass
class PackageInstallPathDeprecation(betterproto.Message):
    """D002"""

    pass


@dataclass
class PackageInstallPathDeprecationMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "PackageInstallPathDeprecation" = betterproto.message_field(2)


@dataclass
class ConfigSourcePathDeprecation(betterproto.Message):
    """D003"""

    deprecated_path: str = betterproto.string_field(1)
    exp_path: str = betterproto.string_field(2)


@dataclass
class ConfigSourcePathDeprecationMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "ConfigSourcePathDeprecation" = betterproto.message_field(2)


@dataclass
class ConfigDataPathDeprecation(betterproto.Message):
    """D004"""

    deprecated_path: str = betterproto.string_field(1)
    exp_path: str = betterproto.string_field(2)


@dataclass
class ConfigDataPathDeprecationMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "ConfigDataPathDeprecation" = betterproto.message_field(2)


@dataclass
class AdapterDeprecationWarning(betterproto.Message):
    """D005"""

    old_name: str = betterproto.string_field(1)
    new_name: str = betterproto.string_field(2)


@dataclass
class AdapterDeprecationWarningMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "AdapterDeprecationWarning" = betterproto.message_field(2)


@dataclass
class MetricAttributesRenamed(betterproto.Message):
    """D006"""

    metric_name: str = betterproto.string_field(1)


@dataclass
class MetricAttributesRenamedMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "MetricAttributesRenamed" = betterproto.message_field(2)


@dataclass
class ExposureNameDeprecation(betterproto.Message):
    """D007"""

    exposure: str = betterproto.string_field(1)


@dataclass
class ExposureNameDeprecationMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "ExposureNameDeprecation" = betterproto.message_field(2)


@dataclass
class InternalDeprecation(betterproto.Message):
    """D008"""

    name: str = betterproto.string_field(1)
    reason: str = betterproto.string_field(2)
    suggested_action: str = betterproto.string_field(3)
    version: str = betterproto.string_field(4)


@dataclass
class InternalDeprecationMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "InternalDeprecation" = betterproto.message_field(2)


@dataclass
class AdapterEventDebug(betterproto.Message):
    """E001"""

    node_info: "NodeInfo" = betterproto.message_field(1)
    name: str = betterproto.string_field(2)
    base_msg: str = betterproto.string_field(3)
    args: List[str] = betterproto.string_field(4)


@dataclass
class AdapterEventDebugMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "AdapterEventDebug" = betterproto.message_field(2)


@dataclass
class AdapterEventInfo(betterproto.Message):
    """E002"""

    node_info: "NodeInfo" = betterproto.message_field(1)
    name: str = betterproto.string_field(2)
    base_msg: str = betterproto.string_field(3)
    args: List[str] = betterproto.string_field(4)


@dataclass
class AdapterEventInfoMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "AdapterEventInfo" = betterproto.message_field(2)


@dataclass
class AdapterEventWarning(betterproto.Message):
    """E003"""

    node_info: "NodeInfo" = betterproto.message_field(1)
    name: str = betterproto.string_field(2)
    base_msg: str = betterproto.string_field(3)
    args: List[str] = betterproto.string_field(4)


@dataclass
class AdapterEventWarningMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "AdapterEventWarning" = betterproto.message_field(2)


@dataclass
class AdapterEventError(betterproto.Message):
    """E004"""

    node_info: "NodeInfo" = betterproto.message_field(1)
    name: str = betterproto.string_field(2)
    base_msg: str = betterproto.string_field(3)
    args: List[str] = betterproto.string_field(4)
    exc_info: str = betterproto.string_field(5)


@dataclass
class AdapterEventErrorMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "AdapterEventError" = betterproto.message_field(2)


@dataclass
class NewConnection(betterproto.Message):
    """E005"""

    node_info: "NodeInfo" = betterproto.message_field(1)
    conn_type: str = betterproto.string_field(2)
    conn_name: str = betterproto.string_field(3)


@dataclass
class NewConnectionMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "NewConnection" = betterproto.message_field(2)


@dataclass
class ConnectionReused(betterproto.Message):
    """E006"""

    conn_name: str = betterproto.string_field(1)


@dataclass
class ConnectionReusedMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "ConnectionReused" = betterproto.message_field(2)


@dataclass
class ConnectionLeftOpenInCleanup(betterproto.Message):
    """E007"""

    conn_name: str = betterproto.string_field(1)


@dataclass
class ConnectionLeftOpenInCleanupMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "ConnectionLeftOpen" = betterproto.message_field(2)


@dataclass
class ConnectionClosedInCleanup(betterproto.Message):
    """E008"""

    conn_name: str = betterproto.string_field(1)


@dataclass
class ConnectionClosedInCleanupMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "ConnectionClosedInCleanup" = betterproto.message_field(2)


@dataclass
class RollbackFailed(betterproto.Message):
    """E009"""

    node_info: "NodeInfo" = betterproto.message_field(1)
    conn_name: str = betterproto.string_field(2)
    exc_info: str = betterproto.string_field(3)


@dataclass
class RollbackFailedMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "RollbackFailed" = betterproto.message_field(2)


@dataclass
class ConnectionClosed(betterproto.Message):
    """E010"""

    node_info: "NodeInfo" = betterproto.message_field(1)
    conn_name: str = betterproto.string_field(2)


@dataclass
class ConnectionClosedMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "ConnectionClosed" = betterproto.message_field(2)


@dataclass
class ConnectionLeftOpen(betterproto.Message):
    """E011"""

    node_info: "NodeInfo" = betterproto.message_field(1)
    conn_name: str = betterproto.string_field(2)


@dataclass
class ConnectionLeftOpenMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "ConnectionLeftOpen" = betterproto.message_field(2)


@dataclass
class Rollback(betterproto.Message):
    """E012"""

    node_info: "NodeInfo" = betterproto.message_field(1)
    conn_name: str = betterproto.string_field(2)


@dataclass
class RollbackMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "Rollback" = betterproto.message_field(2)


@dataclass
class CacheMiss(betterproto.Message):
    """E013"""

    conn_name: str = betterproto.string_field(1)
    database: str = betterproto.string_field(2)
    schema: str = betterproto.string_field(3)


@dataclass
class CacheMissMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "CacheMiss" = betterproto.message_field(2)


@dataclass
class ListRelations(betterproto.Message):
    """E014"""

    database: str = betterproto.string_field(1)
    schema: str = betterproto.string_field(2)
    relations: List["ReferenceKeyMsg"] = betterproto.message_field(3)


@dataclass
class ListRelationsMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "ListRelations" = betterproto.message_field(2)


@dataclass
class ConnectionUsed(betterproto.Message):
    """E015"""

    node_info: "NodeInfo" = betterproto.message_field(1)
    conn_type: str = betterproto.string_field(2)
    conn_name: str = betterproto.string_field(3)


@dataclass
class ConnectionUsedMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "ConnectionUsed" = betterproto.message_field(2)


@dataclass
class SQLQuery(betterproto.Message):
    """E016"""

    node_info: "NodeInfo" = betterproto.message_field(1)
    conn_name: str = betterproto.string_field(2)
    sql: str = betterproto.string_field(3)


@dataclass
class SQLQueryMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "SQLQuery" = betterproto.message_field(2)


@dataclass
class SQLQueryStatus(betterproto.Message):
    """E017"""

    node_info: "NodeInfo" = betterproto.message_field(1)
    status: str = betterproto.string_field(2)
    elapsed: float = betterproto.float_field(3)


@dataclass
class SQLQueryStatusMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "SQLQueryStatus" = betterproto.message_field(2)


@dataclass
class SQLCommit(betterproto.Message):
    """E018"""

    node_info: "NodeInfo" = betterproto.message_field(1)
    conn_name: str = betterproto.string_field(2)


@dataclass
class SQLCommitMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "SQLCommit" = betterproto.message_field(2)


@dataclass
class ColTypeChange(betterproto.Message):
    """E019"""

    orig_type: str = betterproto.string_field(1)
    new_type: str = betterproto.string_field(2)
    table: "ReferenceKeyMsg" = betterproto.message_field(3)


@dataclass
class ColTypeChangeMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "ColTypeChange" = betterproto.message_field(2)


@dataclass
class SchemaCreation(betterproto.Message):
    """E020"""

    relation: "ReferenceKeyMsg" = betterproto.message_field(1)


@dataclass
class SchemaCreationMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "SchemaCreation" = betterproto.message_field(2)


@dataclass
class SchemaDrop(betterproto.Message):
    """E021"""

    relation: "ReferenceKeyMsg" = betterproto.message_field(1)


@dataclass
class SchemaDropMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "SchemaDrop" = betterproto.message_field(2)


@dataclass
class CacheAction(betterproto.Message):
    """E022"""

    action: str = betterproto.string_field(1)
    ref_key: "ReferenceKeyMsg" = betterproto.message_field(2)
    ref_key_2: "ReferenceKeyMsg" = betterproto.message_field(3)
    ref_key_3: "ReferenceKeyMsg" = betterproto.message_field(4)
    ref_list: List["ReferenceKeyMsg"] = betterproto.message_field(5)


@dataclass
class CacheActionMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "CacheAction" = betterproto.message_field(2)


@dataclass
class CacheDumpGraph(betterproto.Message):
    """E031"""

    dump: Dict[str, "ListOfStrings"] = betterproto.map_field(
        1, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )
    before_after: str = betterproto.string_field(2)
    action: str = betterproto.string_field(3)


@dataclass
class CacheDumpGraphMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "CacheDumpGraph" = betterproto.message_field(2)


@dataclass
class AdapterImportError(betterproto.Message):
    """E035"""

    exc: str = betterproto.string_field(1)


@dataclass
class AdapterImportErrorMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "AdapterImportError" = betterproto.message_field(2)


@dataclass
class PluginLoadError(betterproto.Message):
    """E036"""

    exc_info: str = betterproto.string_field(1)


@dataclass
class PluginLoadErrorMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "PluginLoadError" = betterproto.message_field(2)


@dataclass
class NewConnectionOpening(betterproto.Message):
    """E037"""

    node_info: "NodeInfo" = betterproto.message_field(1)
    connection_state: str = betterproto.string_field(2)


@dataclass
class NewConnectionOpeningMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "NewConnectionOpening" = betterproto.message_field(2)


@dataclass
class CodeExecution(betterproto.Message):
    """E038"""

    conn_name: str = betterproto.string_field(1)
    code_content: str = betterproto.string_field(2)


@dataclass
class CodeExecutionMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "CodeExecution" = betterproto.message_field(2)


@dataclass
class CodeExecutionStatus(betterproto.Message):
    """E039"""

    status: str = betterproto.string_field(1)
    elapsed: float = betterproto.float_field(2)


@dataclass
class CodeExecutionStatusMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "CodeExecutionStatus" = betterproto.message_field(2)


@dataclass
class CatalogGenerationError(betterproto.Message):
    """E040"""

    exc: str = betterproto.string_field(1)


@dataclass
class CatalogGenerationErrorMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "CatalogGenerationError" = betterproto.message_field(2)


@dataclass
class WriteCatalogFailure(betterproto.Message):
    """E041"""

    num_exceptions: int = betterproto.int32_field(1)


@dataclass
class WriteCatalogFailureMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "WriteCatalogFailure" = betterproto.message_field(2)


@dataclass
class CatalogWritten(betterproto.Message):
    """E042"""

    path: str = betterproto.string_field(1)


@dataclass
class CatalogWrittenMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "CatalogWritten" = betterproto.message_field(2)


@dataclass
class CannotGenerateDocs(betterproto.Message):
    """E043"""

    pass


@dataclass
class CannotGenerateDocsMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "CannotGenerateDocs" = betterproto.message_field(2)


@dataclass
class BuildingCatalog(betterproto.Message):
    """E044"""

    pass


@dataclass
class BuildingCatalogMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "BuildingCatalog" = betterproto.message_field(2)


@dataclass
class DatabaseErrorRunningHook(betterproto.Message):
    """E045"""

    hook_type: str = betterproto.string_field(1)


@dataclass
class DatabaseErrorRunningHookMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "DatabaseErrorRunningHook" = betterproto.message_field(2)


@dataclass
class HooksRunning(betterproto.Message):
    """E046"""

    num_hooks: int = betterproto.int32_field(1)
    hook_type: str = betterproto.string_field(2)


@dataclass
class HooksRunningMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "HooksRunning" = betterproto.message_field(2)


@dataclass
class FinishedRunningStats(betterproto.Message):
    """E047"""

    stat_line: str = betterproto.string_field(1)
    execution: str = betterproto.string_field(2)
    execution_time: float = betterproto.float_field(3)


@dataclass
class FinishedRunningStatsMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "FinishedRunningStats" = betterproto.message_field(2)


@dataclass
class ParseCmdOut(betterproto.Message):
    """I001"""

    msg: str = betterproto.string_field(1)


@dataclass
class ParseCmdOutMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "ParseCmdOut" = betterproto.message_field(2)


@dataclass
class GenericTestFileParse(betterproto.Message):
    """I011"""

    path: str = betterproto.string_field(1)


@dataclass
class GenericTestFileParseMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "GenericTestFileParse" = betterproto.message_field(2)


@dataclass
class MacroFileParse(betterproto.Message):
    """I012"""

    path: str = betterproto.string_field(1)


@dataclass
class MacroFileParseMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "MacroFileParse" = betterproto.message_field(2)


@dataclass
class PartialParsingErrorProcessingFile(betterproto.Message):
    """I014"""

    file: str = betterproto.string_field(1)


@dataclass
class PartialParsingErrorProcessingFileMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "PartialParsingErrorProcessingFile" = betterproto.message_field(2)


@dataclass
class PartialParsingError(betterproto.Message):
    """I016"""

    exc_info: Dict[str, str] = betterproto.map_field(
        1, betterproto.TYPE_STRING, betterproto.TYPE_STRING
    )


@dataclass
class PartialParsingErrorMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "PartialParsingError" = betterproto.message_field(2)


@dataclass
class PartialParsingSkipParsing(betterproto.Message):
    """I017"""

    pass


@dataclass
class PartialParsingSkipParsingMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "PartialParsingSkipParsing" = betterproto.message_field(2)


@dataclass
class UnableToPartialParse(betterproto.Message):
    """I024"""

    reason: str = betterproto.string_field(1)


@dataclass
class UnableToPartialParseMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "UnableToPartialParse" = betterproto.message_field(2)


@dataclass
class PartialParsingNotEnabled(betterproto.Message):
    """I028"""

    pass


@dataclass
class PartialParsingNotEnabledMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "PartialParsingNotEnabled" = betterproto.message_field(2)


@dataclass
class ParsedFileLoadFailed(betterproto.Message):
    """I029"""

    path: str = betterproto.string_field(1)
    exc: str = betterproto.string_field(2)
    exc_info: str = betterproto.string_field(3)


@dataclass
class ParsedFileLoadFailedMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "ParsedFileLoadFailed" = betterproto.message_field(2)


@dataclass
class StaticParserCausedJinjaRendering(betterproto.Message):
    """I031"""

    path: str = betterproto.string_field(1)


@dataclass
class StaticParserCausedJinjaRenderingMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "StaticParserCausedJinjaRendering" = betterproto.message_field(2)


@dataclass
class UsingExperimentalParser(betterproto.Message):
    """I032"""

    path: str = betterproto.string_field(1)


@dataclass
class UsingExperimentalParserMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "UsingExperimentalParser" = betterproto.message_field(2)


@dataclass
class SampleFullJinjaRendering(betterproto.Message):
    """I033"""

    path: str = betterproto.string_field(1)


@dataclass
class SampleFullJinjaRenderingMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "SampleFullJinjaRendering" = betterproto.message_field(2)


@dataclass
class StaticParserFallbackJinjaRendering(betterproto.Message):
    """I034"""

    path: str = betterproto.string_field(1)


@dataclass
class StaticParserFallbackJinjaRenderingMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "StaticParserFallbackJinjaRendering" = betterproto.message_field(2)


@dataclass
class StaticParsingMacroOverrideDetected(betterproto.Message):
    """I035"""

    path: str = betterproto.string_field(1)


@dataclass
class StaticParsingMacroOverrideDetectedMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "StaticParsingMacroOverrideDetected" = betterproto.message_field(2)


@dataclass
class StaticParserSuccess(betterproto.Message):
    """I036"""

    path: str = betterproto.string_field(1)


@dataclass
class StaticParserSuccessMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "StaticParserSuccess" = betterproto.message_field(2)


@dataclass
class StaticParserFailure(betterproto.Message):
    """I037"""

    path: str = betterproto.string_field(1)


@dataclass
class StaticParserFailureMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "StaticParserFailure" = betterproto.message_field(2)


@dataclass
class ExperimentalParserSuccess(betterproto.Message):
    """I038"""

    path: str = betterproto.string_field(1)


@dataclass
class ExperimentalParserSuccessMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "ExperimentalParserSuccess" = betterproto.message_field(2)


@dataclass
class ExperimentalParserFailure(betterproto.Message):
    """I039"""

    path: str = betterproto.string_field(1)


@dataclass
class ExperimentalParserFailureMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "ExperimentalParserFailure" = betterproto.message_field(2)


@dataclass
class PartialParsingEnabled(betterproto.Message):
    """I040"""

    deleted: int = betterproto.int32_field(1)
    added: int = betterproto.int32_field(2)
    changed: int = betterproto.int32_field(3)


@dataclass
class PartialParsingEnabledMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "PartialParsingEnabled" = betterproto.message_field(2)


@dataclass
class PartialParsingFile(betterproto.Message):
    """I041"""

    file_id: str = betterproto.string_field(1)
    operation: str = betterproto.string_field(2)


@dataclass
class PartialParsingFileMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "PartialParsingFile" = betterproto.message_field(2)


@dataclass
class InvalidDisabledTargetInTestNode(betterproto.Message):
    """I050"""

    resource_type_title: str = betterproto.string_field(1)
    unique_id: str = betterproto.string_field(2)
    original_file_path: str = betterproto.string_field(3)
    target_kind: str = betterproto.string_field(4)
    target_name: str = betterproto.string_field(5)
    target_package: str = betterproto.string_field(6)


@dataclass
class InvalidDisabledTargetInTestNodeMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "InvalidDisabledTargetInTestNode" = betterproto.message_field(2)


@dataclass
class UnusedResourceConfigPath(betterproto.Message):
    """I051"""

    unused_config_paths: List[str] = betterproto.string_field(1)


@dataclass
class UnusedResourceConfigPathMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "UnusedResourceConfigPath" = betterproto.message_field(2)


@dataclass
class SeedIncreased(betterproto.Message):
    """I052"""

    package_name: str = betterproto.string_field(1)
    name: str = betterproto.string_field(2)


@dataclass
class SeedIncreasedMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "SeedIncreased" = betterproto.message_field(2)


@dataclass
class SeedExceedsLimitSamePath(betterproto.Message):
    """I053"""

    package_name: str = betterproto.string_field(1)
    name: str = betterproto.string_field(2)


@dataclass
class SeedExceedsLimitSamePathMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "SeedExceedsLimitSamePath" = betterproto.message_field(2)


@dataclass
class SeedExceedsLimitAndPathChanged(betterproto.Message):
    """I054"""

    package_name: str = betterproto.string_field(1)
    name: str = betterproto.string_field(2)


@dataclass
class SeedExceedsLimitAndPathChangedMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "SeedExceedsLimitAndPathChanged" = betterproto.message_field(2)


@dataclass
class SeedExceedsLimitChecksumChanged(betterproto.Message):
    """I055"""

    package_name: str = betterproto.string_field(1)
    name: str = betterproto.string_field(2)
    checksum_name: str = betterproto.string_field(3)


@dataclass
class SeedExceedsLimitChecksumChangedMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "SeedExceedsLimitChecksumChanged" = betterproto.message_field(2)


@dataclass
class UnusedTables(betterproto.Message):
    """I056"""

    unused_tables: List[str] = betterproto.string_field(1)


@dataclass
class UnusedTablesMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "UnusedTables" = betterproto.message_field(2)


@dataclass
class WrongResourceSchemaFile(betterproto.Message):
    """I057"""

    patch_name: str = betterproto.string_field(1)
    resource_type: str = betterproto.string_field(2)
    plural_resource_type: str = betterproto.string_field(3)
    yaml_key: str = betterproto.string_field(4)
    file_path: str = betterproto.string_field(5)


@dataclass
class WrongResourceSchemaFileMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "WrongResourceSchemaFile" = betterproto.message_field(2)


@dataclass
class NoNodeForYamlKey(betterproto.Message):
    """I058"""

    patch_name: str = betterproto.string_field(1)
    yaml_key: str = betterproto.string_field(2)
    file_path: str = betterproto.string_field(3)


@dataclass
class NoNodeForYamlKeyMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "NoNodeForYamlKey" = betterproto.message_field(2)


@dataclass
class MacroNotFoundForPatch(betterproto.Message):
    """I059"""

    patch_name: str = betterproto.string_field(1)


@dataclass
class MacroNotFoundForPatchMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "MacroNotFoundForPatch" = betterproto.message_field(2)


@dataclass
class NodeNotFoundOrDisabled(betterproto.Message):
    """I060"""

    original_file_path: str = betterproto.string_field(1)
    unique_id: str = betterproto.string_field(2)
    resource_type_title: str = betterproto.string_field(3)
    target_name: str = betterproto.string_field(4)
    target_kind: str = betterproto.string_field(5)
    target_package: str = betterproto.string_field(6)
    disabled: str = betterproto.string_field(7)


@dataclass
class NodeNotFoundOrDisabledMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "NodeNotFoundOrDisabled" = betterproto.message_field(2)


@dataclass
class JinjaLogWarning(betterproto.Message):
    """I061"""

    node_info: "NodeInfo" = betterproto.message_field(1)
    msg: str = betterproto.string_field(2)


@dataclass
class JinjaLogWarningMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "JinjaLogWarning" = betterproto.message_field(2)


@dataclass
class GitSparseCheckoutSubdirectory(betterproto.Message):
    """M001"""

    subdir: str = betterproto.string_field(1)


@dataclass
class GitSparseCheckoutSubdirectoryMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "GitSparseCheckoutSubdirectory" = betterproto.message_field(2)


@dataclass
class GitProgressCheckoutRevision(betterproto.Message):
    """M002"""

    revision: str = betterproto.string_field(1)


@dataclass
class GitProgressCheckoutRevisionMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "GitProgressCheckoutRevision" = betterproto.message_field(2)


@dataclass
class GitProgressUpdatingExistingDependency(betterproto.Message):
    """M003"""

    dir: str = betterproto.string_field(1)


@dataclass
class GitProgressUpdatingExistingDependencyMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "GitProgressUpdatingExistingDependency" = betterproto.message_field(2)


@dataclass
class GitProgressPullingNewDependency(betterproto.Message):
    """M004"""

    dir: str = betterproto.string_field(1)


@dataclass
class GitProgressPullingNewDependencyMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "GitProgressPullingNewDependency" = betterproto.message_field(2)


@dataclass
class GitNothingToDo(betterproto.Message):
    """M005"""

    sha: str = betterproto.string_field(1)


@dataclass
class GitNothingToDoMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "GitNothingToDo" = betterproto.message_field(2)


@dataclass
class GitProgressUpdatedCheckoutRange(betterproto.Message):
    """M006"""

    start_sha: str = betterproto.string_field(1)
    end_sha: str = betterproto.string_field(2)


@dataclass
class GitProgressUpdatedCheckoutRangeMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "GitProgressUpdatedCheckoutRange" = betterproto.message_field(2)


@dataclass
class GitProgressCheckedOutAt(betterproto.Message):
    """M007"""

    end_sha: str = betterproto.string_field(1)


@dataclass
class GitProgressCheckedOutAtMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "GitProgressCheckedOutAt" = betterproto.message_field(2)


@dataclass
class RegistryProgressGETRequest(betterproto.Message):
    """M008"""

    url: str = betterproto.string_field(1)


@dataclass
class RegistryProgressGETRequestMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "RegistryProgressGETRequest" = betterproto.message_field(2)


@dataclass
class RegistryProgressGETResponse(betterproto.Message):
    """M009"""

    url: str = betterproto.string_field(1)
    resp_code: int = betterproto.int32_field(2)


@dataclass
class RegistryProgressGETResponseMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "RegistryProgressGETResponse" = betterproto.message_field(2)


@dataclass
class SelectorReportInvalidSelector(betterproto.Message):
    """M010"""

    valid_selectors: str = betterproto.string_field(1)
    spec_method: str = betterproto.string_field(2)
    raw_spec: str = betterproto.string_field(3)


@dataclass
class SelectorReportInvalidSelectorMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "SelectorReportInvalidSelector" = betterproto.message_field(2)


@dataclass
class JinjaLogInfo(betterproto.Message):
    """M011"""

    node_info: "NodeInfo" = betterproto.message_field(1)
    msg: str = betterproto.string_field(2)


@dataclass
class JinjaLogInfoMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "JinjaLogInfo" = betterproto.message_field(2)


@dataclass
class JinjaLogDebug(betterproto.Message):
    """M012"""

    node_info: "NodeInfo" = betterproto.message_field(1)
    msg: str = betterproto.string_field(2)


@dataclass
class JinjaLogDebugMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "JinjaLogDebug" = betterproto.message_field(2)


@dataclass
class DepsNoPackagesFound(betterproto.Message):
    """M013"""

    pass


@dataclass
class DepsNoPackagesFoundMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "DepsNoPackagesFound" = betterproto.message_field(2)


@dataclass
class DepsStartPackageInstall(betterproto.Message):
    """M014"""

    package_name: str = betterproto.string_field(1)


@dataclass
class DepsStartPackageInstallMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "DepsStartPackageInstall" = betterproto.message_field(2)


@dataclass
class DepsInstallInfo(betterproto.Message):
    """M015"""

    version_name: str = betterproto.string_field(1)


@dataclass
class DepsInstallInfoMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "DepsInstallInfo" = betterproto.message_field(2)


@dataclass
class DepsUpdateAvailable(betterproto.Message):
    """M016"""

    version_latest: str = betterproto.string_field(1)


@dataclass
class DepsUpdateAvailableMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "DepsUpdateAvailable" = betterproto.message_field(2)


@dataclass
class DepsUpToDate(betterproto.Message):
    """M017"""

    pass


@dataclass
class DepsUpToDateMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "DepsUpToDate" = betterproto.message_field(2)


@dataclass
class DepsListSubdirectory(betterproto.Message):
    """M018"""

    subdirectory: str = betterproto.string_field(1)


@dataclass
class DepsListSubdirectoryMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "DepsListSubdirectory" = betterproto.message_field(2)


@dataclass
class DepsNotifyUpdatesAvailable(betterproto.Message):
    """M019"""

    packages: "ListOfStrings" = betterproto.message_field(1)


@dataclass
class DepsNotifyUpdatesAvailableMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "DepsNotifyUpdatesAvailable" = betterproto.message_field(2)


@dataclass
class RetryExternalCall(betterproto.Message):
    """M020"""

    attempt: int = betterproto.int32_field(1)
    max: int = betterproto.int32_field(2)


@dataclass
class RetryExternalCallMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "RetryExternalCall" = betterproto.message_field(2)


@dataclass
class RecordRetryException(betterproto.Message):
    """M021"""

    exc: str = betterproto.string_field(1)


@dataclass
class RecordRetryExceptionMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "RecordRetryException" = betterproto.message_field(2)


@dataclass
class RegistryIndexProgressGETRequest(betterproto.Message):
    """M022"""

    url: str = betterproto.string_field(1)


@dataclass
class RegistryIndexProgressGETRequestMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "RegistryIndexProgressGETRequest" = betterproto.message_field(2)


@dataclass
class RegistryIndexProgressGETResponse(betterproto.Message):
    """M023"""

    url: str = betterproto.string_field(1)
    resp_code: int = betterproto.int32_field(2)


@dataclass
class RegistryIndexProgressGETResponseMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "RegistryIndexProgressGETResponse" = betterproto.message_field(2)


@dataclass
class RegistryResponseUnexpectedType(betterproto.Message):
    """M024"""

    response: str = betterproto.string_field(1)


@dataclass
class RegistryResponseUnexpectedTypeMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "RegistryResponseUnexpectedType" = betterproto.message_field(2)


@dataclass
class RegistryResponseMissingTopKeys(betterproto.Message):
    """M025"""

    response: str = betterproto.string_field(1)


@dataclass
class RegistryResponseMissingTopKeysMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "RegistryResponseMissingTopKeys" = betterproto.message_field(2)


@dataclass
class RegistryResponseMissingNestedKeys(betterproto.Message):
    """M026"""

    response: str = betterproto.string_field(1)


@dataclass
class RegistryResponseMissingNestedKeysMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "RegistryResponseMissingNestedKeys" = betterproto.message_field(2)


@dataclass
class RegistryResponseExtraNestedKeys(betterproto.Message):
    """m027"""

    response: str = betterproto.string_field(1)


@dataclass
class RegistryResponseExtraNestedKeysMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "RegistryResponseExtraNestedKeys" = betterproto.message_field(2)


@dataclass
class DepsSetDownloadDirectory(betterproto.Message):
    """M028"""

    path: str = betterproto.string_field(1)


@dataclass
class DepsSetDownloadDirectoryMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "DepsSetDownloadDirectory" = betterproto.message_field(2)


@dataclass
class DepsUnpinned(betterproto.Message):
    """M029"""

    revision: str = betterproto.string_field(1)
    git: str = betterproto.string_field(2)


@dataclass
class DepsUnpinnedMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "DepsUnpinned" = betterproto.message_field(2)


@dataclass
class NoNodesForSelectionCriteria(betterproto.Message):
    """M030"""

    spec_raw: str = betterproto.string_field(1)


@dataclass
class NoNodesForSelectionCriteriaMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "NoNodesForSelectionCriteria" = betterproto.message_field(2)


@dataclass
class RunningOperationCaughtError(betterproto.Message):
    """Q001"""

    exc: str = betterproto.string_field(1)


@dataclass
class RunningOperationCaughtErrorMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "RunningOperationCaughtError" = betterproto.message_field(2)


@dataclass
class CompileComplete(betterproto.Message):
    """Q002"""

    pass


@dataclass
class CompileCompleteMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "CompileComplete" = betterproto.message_field(2)


@dataclass
class FreshnessCheckComplete(betterproto.Message):
    """Q003"""

    pass


@dataclass
class FreshnessCheckCompleteMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "FreshnessCheckComplete" = betterproto.message_field(2)


@dataclass
class SeedHeader(betterproto.Message):
    """Q004"""

    header: str = betterproto.string_field(1)


@dataclass
class SeedHeaderMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "SeedHeader" = betterproto.message_field(2)


@dataclass
class SeedHeaderSeparator(betterproto.Message):
    """Q005"""

    len_header: int = betterproto.int32_field(1)


@dataclass
class SeedHeaderSeparatorMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "SeedHeaderSeparator" = betterproto.message_field(2)


@dataclass
class SQLRunnerException(betterproto.Message):
    """Q006"""

    exc: str = betterproto.string_field(1)
    exc_info: str = betterproto.string_field(2)


@dataclass
class SQLRunnerExceptionMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "SQLRunnerException" = betterproto.message_field(2)


@dataclass
class LogTestResult(betterproto.Message):
    """Q007"""

    node_info: "NodeInfo" = betterproto.message_field(1)
    name: str = betterproto.string_field(2)
    status: str = betterproto.string_field(3)
    index: int = betterproto.int32_field(4)
    num_models: int = betterproto.int32_field(5)
    execution_time: float = betterproto.float_field(6)
    num_failures: int = betterproto.int32_field(7)


@dataclass
class LogTestResultMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "LogTestResult" = betterproto.message_field(2)


@dataclass
class LogStartLine(betterproto.Message):
    """Q011"""

    node_info: "NodeInfo" = betterproto.message_field(1)
    description: str = betterproto.string_field(2)
    index: int = betterproto.int32_field(3)
    total: int = betterproto.int32_field(4)


@dataclass
class LogStartLineMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "LogStartLine" = betterproto.message_field(2)


@dataclass
class LogModelResult(betterproto.Message):
    """Q012"""

    node_info: "NodeInfo" = betterproto.message_field(1)
    description: str = betterproto.string_field(2)
    status: str = betterproto.string_field(3)
    index: int = betterproto.int32_field(4)
    total: int = betterproto.int32_field(5)
    execution_time: int = betterproto.int32_field(6)


@dataclass
class LogModelResultMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "LogModelResult" = betterproto.message_field(2)


@dataclass
class LogSnapshotResult(betterproto.Message):
    """Q015"""

    node_info: "NodeInfo" = betterproto.message_field(1)
    description: str = betterproto.string_field(2)
    status: str = betterproto.string_field(3)
    index: int = betterproto.int32_field(4)
    total: int = betterproto.int32_field(5)
    execution_time: float = betterproto.float_field(6)
    cfg: Dict[str, str] = betterproto.map_field(
        7, betterproto.TYPE_STRING, betterproto.TYPE_STRING
    )


@dataclass
class LogSnapshotResultMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "LogSnapshotResult" = betterproto.message_field(2)


@dataclass
class LogSeedResult(betterproto.Message):
    """Q016"""

    node_info: "NodeInfo" = betterproto.message_field(1)
    status: str = betterproto.string_field(2)
    result_message: str = betterproto.string_field(3)
    index: int = betterproto.int32_field(4)
    total: int = betterproto.int32_field(5)
    execution_time: float = betterproto.float_field(6)
    schema: str = betterproto.string_field(7)
    relation: str = betterproto.string_field(8)


@dataclass
class LogSeedResultMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "LogSeedResult" = betterproto.message_field(2)


@dataclass
class LogFreshnessResult(betterproto.Message):
    """Q018"""

    status: str = betterproto.string_field(1)
    node_info: "NodeInfo" = betterproto.message_field(2)
    index: int = betterproto.int32_field(3)
    total: int = betterproto.int32_field(4)
    execution_time: float = betterproto.float_field(5)
    source_name: str = betterproto.string_field(6)
    table_name: str = betterproto.string_field(7)


@dataclass
class LogFreshnessResultMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "LogFreshnessResult" = betterproto.message_field(2)


@dataclass
class LogCancelLine(betterproto.Message):
    """Q022"""

    conn_name: str = betterproto.string_field(1)


@dataclass
class LogCancelLineMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "LogCancelLine" = betterproto.message_field(2)


@dataclass
class DefaultSelector(betterproto.Message):
    """Q023"""

    name: str = betterproto.string_field(1)


@dataclass
class DefaultSelectorMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "DefaultSelector" = betterproto.message_field(2)


@dataclass
class NodeStart(betterproto.Message):
    """Q024"""

    node_info: "NodeInfo" = betterproto.message_field(1)


@dataclass
class NodeStartMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "NodeStart" = betterproto.message_field(2)


@dataclass
class NodeFinished(betterproto.Message):
    """Q025"""

    node_info: "NodeInfo" = betterproto.message_field(1)
    run_result: "RunResultMsg" = betterproto.message_field(2)


@dataclass
class NodeFinishedMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "NodeFinished" = betterproto.message_field(2)


@dataclass
class QueryCancelationUnsupported(betterproto.Message):
    """Q026"""

    type: str = betterproto.string_field(1)


@dataclass
class QueryCancelationUnsupportedMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "QueryCancelationUnsupported" = betterproto.message_field(2)


@dataclass
class ConcurrencyLine(betterproto.Message):
    """Q027"""

    num_threads: int = betterproto.int32_field(1)
    target_name: str = betterproto.string_field(2)
    node_count: int = betterproto.int32_field(3)


@dataclass
class ConcurrencyLineMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "ConcurrencyLine" = betterproto.message_field(2)


@dataclass
class WritingInjectedSQLForNode(betterproto.Message):
    """Q029"""

    node_info: "NodeInfo" = betterproto.message_field(1)


@dataclass
class WritingInjectedSQLForNodeMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "WritingInjectedSQLForNode" = betterproto.message_field(2)


@dataclass
class NodeCompiling(betterproto.Message):
    """Q030"""

    node_info: "NodeInfo" = betterproto.message_field(1)


@dataclass
class NodeCompilingMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "NodeCompiling" = betterproto.message_field(2)


@dataclass
class NodeExecuting(betterproto.Message):
    """Q031"""

    node_info: "NodeInfo" = betterproto.message_field(1)


@dataclass
class NodeExecutingMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "NodeExecuting" = betterproto.message_field(2)


@dataclass
class LogHookStartLine(betterproto.Message):
    """Q032"""

    node_info: "NodeInfo" = betterproto.message_field(1)
    statement: str = betterproto.string_field(2)
    index: int = betterproto.int32_field(3)
    total: int = betterproto.int32_field(4)


@dataclass
class LogHookStartLineMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "LogHookStartLine" = betterproto.message_field(2)


@dataclass
class LogHookEndLine(betterproto.Message):
    """Q033"""

    node_info: "NodeInfo" = betterproto.message_field(1)
    statement: str = betterproto.string_field(2)
    status: str = betterproto.string_field(3)
    index: int = betterproto.int32_field(4)
    total: int = betterproto.int32_field(5)
    execution_time: float = betterproto.float_field(6)


@dataclass
class LogHookEndLineMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "LogHookEndLine" = betterproto.message_field(2)


@dataclass
class SkippingDetails(betterproto.Message):
    """Q034"""

    node_info: "NodeInfo" = betterproto.message_field(1)
    resource_type: str = betterproto.string_field(2)
    schema: str = betterproto.string_field(3)
    node_name: str = betterproto.string_field(4)
    index: int = betterproto.int32_field(5)
    total: int = betterproto.int32_field(6)


@dataclass
class SkippingDetailsMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "SkippingDetails" = betterproto.message_field(2)


@dataclass
class NothingToDo(betterproto.Message):
    """Q035"""

    pass


@dataclass
class NothingToDoMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "NothingToDo" = betterproto.message_field(2)


@dataclass
class RunningOperationUncaughtError(betterproto.Message):
    """Q036"""

    exc: str = betterproto.string_field(1)


@dataclass
class RunningOperationUncaughtErrorMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "RunningOperationUncaughtError" = betterproto.message_field(2)


@dataclass
class EndRunResult(betterproto.Message):
    """Q037"""

    results: List["RunResultMsg"] = betterproto.message_field(1)
    elapsed_time: float = betterproto.float_field(2)
    generated_at: datetime = betterproto.message_field(3)
    success: bool = betterproto.bool_field(4)


@dataclass
class EndRunResultMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "EndRunResult" = betterproto.message_field(2)


@dataclass
class NoNodesSelected(betterproto.Message):
    """Q038"""

    pass


@dataclass
class NoNodesSelectedMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "NoNodesSelected" = betterproto.message_field(2)


@dataclass
class CatchableExceptionOnRun(betterproto.Message):
    """W002"""

    node_info: "NodeInfo" = betterproto.message_field(1)
    exc: str = betterproto.string_field(2)
    exc_info: str = betterproto.string_field(3)


@dataclass
class CatchableExceptionOnRunMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "CatchableExceptionOnRun" = betterproto.message_field(2)


@dataclass
class InternalErrorOnRun(betterproto.Message):
    """W003"""

    build_path: str = betterproto.string_field(1)
    exc: str = betterproto.string_field(2)


@dataclass
class InternalErrorOnRunMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "InternalErrorOnRun" = betterproto.message_field(2)


@dataclass
class GenericExceptionOnRun(betterproto.Message):
    """W004"""

    build_path: str = betterproto.string_field(1)
    unique_id: str = betterproto.string_field(2)
    exc: str = betterproto.string_field(3)


@dataclass
class GenericExceptionOnRunMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "GenericExceptionOnRun" = betterproto.message_field(2)


@dataclass
class NodeConnectionReleaseError(betterproto.Message):
    """W005"""

    node_name: str = betterproto.string_field(1)
    exc: str = betterproto.string_field(2)
    exc_info: str = betterproto.string_field(3)


@dataclass
class NodeConnectionReleaseErrorMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "NodeConnectionReleaseError" = betterproto.message_field(2)


@dataclass
class FoundStats(betterproto.Message):
    """W006"""

    stat_line: str = betterproto.string_field(1)


@dataclass
class FoundStatsMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "FoundStats" = betterproto.message_field(2)


@dataclass
class MainKeyboardInterrupt(betterproto.Message):
    """Z001"""

    pass


@dataclass
class MainKeyboardInterruptMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "MainKeyboardInterrupt" = betterproto.message_field(2)


@dataclass
class MainEncounteredError(betterproto.Message):
    """Z002"""

    exc: str = betterproto.string_field(1)


@dataclass
class MainEncounteredErrorMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "MainEncounteredError" = betterproto.message_field(2)


@dataclass
class MainStackTrace(betterproto.Message):
    """Z003"""

    stack_trace: str = betterproto.string_field(1)


@dataclass
class MainStackTraceMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "MainStackTrace" = betterproto.message_field(2)


@dataclass
class SystemErrorRetrievingModTime(betterproto.Message):
    """Z004"""

    path: str = betterproto.string_field(1)


@dataclass
class SystemErrorRetrievingModTimeMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "SystemErrorRetrievingModTime" = betterproto.message_field(2)


@dataclass
class SystemCouldNotWrite(betterproto.Message):
    """Z005"""

    path: str = betterproto.string_field(1)
    reason: str = betterproto.string_field(2)
    exc: str = betterproto.string_field(3)


@dataclass
class SystemCouldNotWriteMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "SystemCouldNotWrite" = betterproto.message_field(2)


@dataclass
class SystemExecutingCmd(betterproto.Message):
    """Z006"""

    cmd: List[str] = betterproto.string_field(1)


@dataclass
class SystemExecutingCmdMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "SystemExecutingCmd" = betterproto.message_field(2)


@dataclass
class SystemStdOut(betterproto.Message):
    """Z007"""

    bmsg: bytes = betterproto.bytes_field(1)


@dataclass
class SystemStdOutMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "SystemStdOut" = betterproto.message_field(2)


@dataclass
class SystemStdErr(betterproto.Message):
    """Z008"""

    bmsg: bytes = betterproto.bytes_field(1)


@dataclass
class SystemStdErrMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "SystemStdErr" = betterproto.message_field(2)


@dataclass
class SystemReportReturnCode(betterproto.Message):
    """Z009"""

    returncode: int = betterproto.int32_field(1)


@dataclass
class SystemReportReturnCodeMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "SystemReportReturnCode" = betterproto.message_field(2)


@dataclass
class TimingInfoCollected(betterproto.Message):
    """Z010"""

    node_info: "NodeInfo" = betterproto.message_field(1)
    timing_info: "TimingInfoMsg" = betterproto.message_field(2)


@dataclass
class TimingInfoCollectedMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "TimingInfoCollected" = betterproto.message_field(2)


@dataclass
class LogDebugStackTrace(betterproto.Message):
    """Z011"""

    exc_info: str = betterproto.string_field(1)


@dataclass
class LogDebugStackTraceMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "LogDebugStackTrace" = betterproto.message_field(2)


@dataclass
class CheckCleanPath(betterproto.Message):
    """Z012"""

    path: str = betterproto.string_field(1)


@dataclass
class CheckCleanPathMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "CheckCleanPath" = betterproto.message_field(2)


@dataclass
class ConfirmCleanPath(betterproto.Message):
    """Z013"""

    path: str = betterproto.string_field(1)


@dataclass
class ConfirmCleanPathMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "ConfirmCleanPath" = betterproto.message_field(2)


@dataclass
class ProtectedCleanPath(betterproto.Message):
    """Z014"""

    path: str = betterproto.string_field(1)


@dataclass
class ProtectedCleanPathMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "ProtectedCleanPath" = betterproto.message_field(2)


@dataclass
class FinishedCleanPaths(betterproto.Message):
    """Z015"""

    pass


@dataclass
class FinishedCleanPathsMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "FinishedCleanPaths" = betterproto.message_field(2)


@dataclass
class OpenCommand(betterproto.Message):
    """Z016"""

    open_cmd: str = betterproto.string_field(1)
    profiles_dir: str = betterproto.string_field(2)


@dataclass
class OpenCommandMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "OpenCommand" = betterproto.message_field(2)


@dataclass
class EmptyLine(betterproto.Message):
    """Z017"""

    pass


@dataclass
class EmptyLineMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "EmptyLine" = betterproto.message_field(2)


@dataclass
class ServingDocsPort(betterproto.Message):
    """Z018"""

    address: str = betterproto.string_field(1)
    port: int = betterproto.int32_field(2)


@dataclass
class ServingDocsPortMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "ServingDocsPort" = betterproto.message_field(2)


@dataclass
class ServingDocsAccessInfo(betterproto.Message):
    """Z019"""

    port: str = betterproto.string_field(1)


@dataclass
class ServingDocsAccessInfoMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "ServingDocsAccessInfo" = betterproto.message_field(2)


@dataclass
class ServingDocsExitInfo(betterproto.Message):
    """Z020"""

    pass


@dataclass
class ServingDocsExitInfoMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "ServingDocsExitInfo" = betterproto.message_field(2)


@dataclass
class RunResultWarning(betterproto.Message):
    """Z021"""

    resource_type: str = betterproto.string_field(1)
    node_name: str = betterproto.string_field(2)
    path: str = betterproto.string_field(3)


@dataclass
class RunResultWarningMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "RunResultWarning" = betterproto.message_field(2)


@dataclass
class RunResultFailure(betterproto.Message):
    """Z022"""

    resource_type: str = betterproto.string_field(1)
    node_name: str = betterproto.string_field(2)
    path: str = betterproto.string_field(3)


@dataclass
class RunResultFailureMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "RunResultFailure" = betterproto.message_field(2)


@dataclass
class StatsLine(betterproto.Message):
    """Z023"""

    stats: Dict[str, int] = betterproto.map_field(
        1, betterproto.TYPE_STRING, betterproto.TYPE_INT32
    )


@dataclass
class StatsLineMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "StatsLine" = betterproto.message_field(2)


@dataclass
class RunResultError(betterproto.Message):
    """Z024"""

    msg: str = betterproto.string_field(1)


@dataclass
class RunResultErrorMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "RunResultError" = betterproto.message_field(2)


@dataclass
class RunResultErrorNoMessage(betterproto.Message):
    """Z025"""

    status: str = betterproto.string_field(1)


@dataclass
class RunResultErrorNoMessageMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "RunResultErrorNoMessage" = betterproto.message_field(2)


@dataclass
class SQLCompiledPath(betterproto.Message):
    """Z026"""

    path: str = betterproto.string_field(1)


@dataclass
class SQLCompiledPathMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "SQLCompiledPath" = betterproto.message_field(2)


@dataclass
class CheckNodeTestFailure(betterproto.Message):
    """Z027"""

    relation_name: str = betterproto.string_field(1)


@dataclass
class CheckNodeTestFailureMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "CheckNodeTestFailure" = betterproto.message_field(2)


@dataclass
class FirstRunResultError(betterproto.Message):
    """Z028"""

    msg: str = betterproto.string_field(1)


@dataclass
class FirstRunResultErrorMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "FirstRunResultError" = betterproto.message_field(2)


@dataclass
class AfterFirstRunResultError(betterproto.Message):
    """Z029"""

    msg: str = betterproto.string_field(1)


@dataclass
class AfterFirstRunResultErrorMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "AfterFirstRunResultError" = betterproto.message_field(2)


@dataclass
class EndOfRunSummary(betterproto.Message):
    """Z030"""

    num_errors: int = betterproto.int32_field(1)
    num_warnings: int = betterproto.int32_field(2)
    keyboard_interrupt: bool = betterproto.bool_field(3)


@dataclass
class EndOfRunSummaryMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "EndOfRunSummary" = betterproto.message_field(2)


@dataclass
class LogSkipBecauseError(betterproto.Message):
    """Z034"""

    schema: str = betterproto.string_field(1)
    relation: str = betterproto.string_field(2)
    index: int = betterproto.int32_field(3)
    total: int = betterproto.int32_field(4)


@dataclass
class LogSkipBecauseErrorMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "LogSkipBecauseError" = betterproto.message_field(2)


@dataclass
class EnsureGitInstalled(betterproto.Message):
    """Z036"""

    pass


@dataclass
class EnsureGitInstalledMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "EnsureGitInstalled" = betterproto.message_field(2)


@dataclass
class DepsCreatingLocalSymlink(betterproto.Message):
    """Z037"""

    pass


@dataclass
class DepsCreatingLocalSymlinkMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "DepsCreatingLocalSymlink" = betterproto.message_field(2)


@dataclass
class DepsSymlinkNotAvailable(betterproto.Message):
    """Z038"""

    pass


@dataclass
class DepsSymlinkNotAvailableMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "DepsSymlinkNotAvailable" = betterproto.message_field(2)


@dataclass
class DisableTracking(betterproto.Message):
    """Z039"""

    pass


@dataclass
class DisableTrackingMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "DisableTracking" = betterproto.message_field(2)


@dataclass
class SendingEvent(betterproto.Message):
    """Z040"""

    kwargs: str = betterproto.string_field(1)


@dataclass
class SendingEventMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "SendingEvent" = betterproto.message_field(2)


@dataclass
class SendEventFailure(betterproto.Message):
    """Z041"""

    pass


@dataclass
class SendEventFailureMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "SendEventFailure" = betterproto.message_field(2)


@dataclass
class FlushEvents(betterproto.Message):
    """Z042"""

    pass


@dataclass
class FlushEventsMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "FlushEvents" = betterproto.message_field(2)


@dataclass
class FlushEventsFailure(betterproto.Message):
    """Z043"""

    pass


@dataclass
class FlushEventsFailureMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "FlushEventsFailure" = betterproto.message_field(2)


@dataclass
class TrackingInitializeFailure(betterproto.Message):
    """Z044"""

    exc_info: str = betterproto.string_field(1)


@dataclass
class TrackingInitializeFailureMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "TrackingInitializeFailure" = betterproto.message_field(2)


@dataclass
class RunResultWarningMessage(betterproto.Message):
    """Z046"""

    msg: str = betterproto.string_field(1)


@dataclass
class RunResultWarningMessageMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "RunResultWarningMessage" = betterproto.message_field(2)


@dataclass
class IntegrationTestInfo(betterproto.Message):
    """T001"""

    msg: str = betterproto.string_field(1)


@dataclass
class IntegrationTestInfoMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "IntegrationTestInfo" = betterproto.message_field(2)


@dataclass
class IntegrationTestDebug(betterproto.Message):
    """T002"""

    msg: str = betterproto.string_field(1)


@dataclass
class IntegrationTestDebugMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "IntegrationTestDebug" = betterproto.message_field(2)


@dataclass
class IntegrationTestWarn(betterproto.Message):
    """T003"""

    msg: str = betterproto.string_field(1)


@dataclass
class IntegrationTestWarnMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "IntegrationTestWarn" = betterproto.message_field(2)


@dataclass
class IntegrationTestError(betterproto.Message):
    """T004"""

    msg: str = betterproto.string_field(1)


@dataclass
class IntegrationTestErrorMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "IntegrationTestError" = betterproto.message_field(2)


@dataclass
class IntegrationTestException(betterproto.Message):
    """T005"""

    msg: str = betterproto.string_field(1)


@dataclass
class IntegrationTestExceptionMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "IntegrationTestException" = betterproto.message_field(2)


@dataclass
class UnitTestInfo(betterproto.Message):
    """T006"""

    msg: str = betterproto.string_field(1)


@dataclass
class UnitTestInfoMsg(betterproto.Message):
    info: "EventInfo" = betterproto.message_field(1)
    data: "UnitTestInfo" = betterproto.message_field(2)



import betterproto
from dbt.constants import METADATA_ENV_PREFIX
from dbt.events.base_types import BaseEvent, Cache, EventLevel, NoFile, NoStdOut, EventMsg
from dbt.events.eventmgr import EventManager, LoggerConfig, LineFormat, NoFilter
from dbt.events.helpers import env_secrets, scrub_secrets
from dbt.events.types import EmptyLine
import dbt.flags as flags
from dbt.logger import GLOBAL_LOGGER, make_log_dir_if_missing
from functools import partial
import json
import os
import sys
from typing import Callable, Dict, Optional, TextIO
import uuid


LOG_VERSION = 3
metadata_vars: Optional[Dict[str, str]] = None


def setup_event_logger(log_path: str, level_override: Optional[EventLevel] = None):
    cleanup_event_logger()
    make_log_dir_if_missing(log_path)
    if flags.ENABLE_LEGACY_LOGGER:
        EVENT_MANAGER.add_logger(_get_logbook_log_config(level_override))
    else:
        EVENT_MANAGER.add_logger(_get_stdout_config(level_override))

        if _CAPTURE_STREAM:
            # Create second stdout logger to support test which want to know what's
            # being sent to stdout.
            capture_config = _get_stdout_config(level_override)
            capture_config.output_stream = _CAPTURE_STREAM
            EVENT_MANAGER.add_logger(capture_config)

        # create and add the file logger to the event manager
        EVENT_MANAGER.add_logger(_get_logfile_config(os.path.join(log_path, "dbt.log")))


def _get_stdout_config(level: Optional[EventLevel] = None) -> LoggerConfig:
    fmt = LineFormat.PlainText
    if flags.LOG_FORMAT == "json":
        fmt = LineFormat.Json
    elif flags.DEBUG:
        fmt = LineFormat.DebugText

    return LoggerConfig(
        name="stdout_log",
        level=level or (EventLevel.DEBUG if flags.DEBUG else EventLevel.INFO),
        use_colors=bool(flags.USE_COLORS),
        line_format=fmt,
        scrubber=env_scrubber,
        filter=partial(
            _stdout_filter, bool(flags.LOG_CACHE_EVENTS), bool(flags.DEBUG), bool(flags.QUIET)
        ),
        output_stream=sys.stdout,
    )


def _stdout_filter(
    log_cache_events: bool, debug_mode: bool, quiet_mode: bool, msg: EventMsg
) -> bool:
    return (
        not isinstance(msg.data, NoStdOut)
        and (not isinstance(msg.data, Cache) or log_cache_events)
        and (EventLevel(msg.info.level) != EventLevel.DEBUG or debug_mode)
        and (EventLevel(msg.info.level) == EventLevel.ERROR or not quiet_mode)
        and not (flags.LOG_FORMAT == "json" and type(msg.data) == EmptyLine)
    )


def _get_logfile_config(log_path: str) -> LoggerConfig:
    return LoggerConfig(
        name="file_log",
        line_format=LineFormat.Json if flags.LOG_FORMAT == "json" else LineFormat.DebugText,
        use_colors=bool(flags.USE_COLORS),
        level=EventLevel.DEBUG,  # File log is *always* debug level
        scrubber=env_scrubber,
        filter=partial(_logfile_filter, bool(flags.LOG_CACHE_EVENTS)),
        output_file_name=log_path,
    )


def _logfile_filter(log_cache_events: bool, msg: EventMsg) -> bool:
    return (
        not isinstance(msg.data, NoFile)
        and not (isinstance(msg.data, Cache) and not log_cache_events)
        and not (flags.LOG_FORMAT == "json" and type(msg.data) == EmptyLine)
    )


def _get_logbook_log_config(level: Optional[EventLevel] = None) -> LoggerConfig:
    config = _get_stdout_config(level)
    config.name = "logbook_log"
    config.filter = NoFilter if flags.LOG_CACHE_EVENTS else lambda e: not isinstance(e.data, Cache)
    config.logger = GLOBAL_LOGGER
    return config


def env_scrubber(msg: str) -> str:
    return scrub_secrets(msg, env_secrets())


def cleanup_event_logger():
    # Reset to a no-op manager to release streams associated with logs. This is
    # especially important for tests, since pytest replaces the stdout stream
    # during test runs, and closes the stream after the test is over.
    EVENT_MANAGER.loggers.clear()
    EVENT_MANAGER.callbacks.clear()


# Since dbt-rpc does not do its own log setup, and since some events can
# currently fire before logs can be configured by setup_event_logger(), we
# create a default configuration with default settings and no file output.
EVENT_MANAGER: EventManager = EventManager()
EVENT_MANAGER.add_logger(
    _get_logbook_log_config() if flags.ENABLE_LEGACY_LOGGER else _get_stdout_config()
)


# This global, and the following two functions for capturing stdout logs are
# an unpleasant hack we intend to remove as part of API-ification. The GitHub
# issue #6350 was opened for that work.
_CAPTURE_STREAM: Optional[TextIO] = None


# used for integration tests
def capture_stdout_logs(stream: TextIO):
    global _CAPTURE_STREAM
    _CAPTURE_STREAM = stream


def stop_capture_stdout_logs():
    global _CAPTURE_STREAM
    _CAPTURE_STREAM = None


# returns a dictionary representation of the event fields.
# the message may contain secrets which must be scrubbed at the usage site.
def msg_to_json(msg: EventMsg) -> str:
    msg_dict = msg_to_dict(msg)
    raw_log_line = json.dumps(msg_dict, sort_keys=True)
    return raw_log_line


def msg_to_dict(msg: EventMsg) -> dict:
    msg_dict = dict()
    try:
        msg_dict = msg.to_dict(casing=betterproto.Casing.SNAKE, include_default_values=True)  # type: ignore
    except AttributeError as exc:
        event_type = type(msg).__name__
        raise Exception(f"type {event_type} is not serializable. {str(exc)}")
    # We don't want an empty NodeInfo in output
    if (
        "data" in msg_dict
        and "node_info" in msg_dict["data"]
        and msg_dict["data"]["node_info"]["node_name"] == ""
    ):
        del msg_dict["data"]["node_info"]
    return msg_dict


def warn_or_error(event, node=None):
    # TODO: resolve this circular import when flags.WARN_ERROR_OPTIONS is WarnErrorOptions type via click CLI.
    from dbt.helper_types import WarnErrorOptions

    warn_error_options = WarnErrorOptions.from_yaml_string(flags.WARN_ERROR_OPTIONS)
    if flags.WARN_ERROR or warn_error_options.includes(type(event).__name__):
        # TODO: resolve this circular import when at top
        from dbt.exceptions import EventCompilationError

        raise EventCompilationError(event.message(), node)
    else:
        fire_event(event)


# an alternative to fire_event which only creates and logs the event value
# if the condition is met. Does nothing otherwise.
def fire_event_if(
    conditional: bool, lazy_e: Callable[[], BaseEvent], level: EventLevel = None
) -> None:
    if conditional:
        fire_event(lazy_e(), level=level)


# top-level method for accessing the new eventing system
# this is where all the side effects happen branched by event type
# (i.e. - mutating the event history, printing to stdout, logging
# to files, etc.)
def fire_event(e: BaseEvent, level: EventLevel = None) -> None:
    EVENT_MANAGER.fire_event(e, level=level)


def get_metadata_vars() -> Dict[str, str]:
    global metadata_vars
    if metadata_vars is None:
        metadata_vars = {
            k[len(METADATA_ENV_PREFIX) :]: v
            for k, v in os.environ.items()
            if k.startswith(METADATA_ENV_PREFIX)
        }
    return metadata_vars


def reset_metadata_vars() -> None:
    global metadata_vars
    metadata_vars = None


def get_invocation_id() -> str:
    return EVENT_MANAGER.invocation_id


def set_invocation_id() -> None:
    # This is primarily for setting the invocation_id for separate
    # commands in the dbt servers. It shouldn't be necessary for the CLI.
    EVENT_MANAGER.invocation_id = str(uuid.uuid4())



import inspect  # This is temporary for RAT-ing
from copy import copy
from pprint import pformat as pf  # This is temporary for RAT-ing

import click
from dbt.adapters.factory import adapter_management
from dbt.cli import params as p
from dbt.cli.flags import Flags
from dbt.profiler import profiler


def cli_runner():
    # Alias "list" to "ls"
    ls = copy(cli.commands["list"])
    ls.hidden = True
    cli.add_command(ls, "ls")

    # Run the cli
    cli()


# dbt
@click.group(
    context_settings={"help_option_names": ["-h", "--help"]},
    invoke_without_command=True,
    no_args_is_help=True,
    epilog="Specify one of these sub-commands and you can find more help from there.",
)
@click.pass_context
@p.anonymous_usage_stats
@p.cache_selected_only
@p.debug
@p.enable_legacy_logger
@p.fail_fast
@p.log_cache_events
@p.log_format
@p.macro_debugging
@p.partial_parse
@p.print
@p.printer_width
@p.quiet
@p.record_timing_info
@p.static_parser
@p.use_colors
@p.use_experimental_parser
@p.version
@p.version_check
@p.warn_error
@p.warn_error_options
@p.write_json
def cli(ctx, **kwargs):
    """An ELT tool for managing your SQL transformations and data models.
    For more documentation on these commands, visit: docs.getdbt.com
    """
    incomplete_flags = Flags()

    # Profiling
    if incomplete_flags.RECORD_TIMING_INFO:
        ctx.with_resource(profiler(enable=True, outfile=incomplete_flags.RECORD_TIMING_INFO))

    # Adapter management
    ctx.with_resource(adapter_management())

    # Version info
    if incomplete_flags.VERSION:
        click.echo(f"`version` called\n ctx.params: {pf(ctx.params)}")
        return
    else:
        del ctx.params["version"]


# dbt build
@cli.command("build")
@click.pass_context
@p.defer
@p.exclude
@p.fail_fast
@p.full_refresh
@p.indirect_selection
@p.log_path
@p.models
@p.profile
@p.profiles_dir
@p.project_dir
@p.selector
@p.show
@p.state
@p.store_failures
@p.target
@p.target_path
@p.threads
@p.vars
@p.version_check
def build(ctx, **kwargs):
    """Run all Seeds, Models, Snapshots, and tests in DAG order"""
    flags = Flags()
    click.echo(f"`{inspect.stack()[0][3]}` called\n flags: {flags}")


# dbt clean
@cli.command("clean")
@click.pass_context
@p.profile
@p.profiles_dir
@p.project_dir
@p.target
@p.vars
def clean(ctx, **kwargs):
    """Delete all folders in the clean-targets list (usually the dbt_packages and target directories.)"""
    flags = Flags()
    click.echo(f"`{inspect.stack()[0][3]}` called\n flags: {flags}")


# dbt docs
@cli.group()
@click.pass_context
def docs(ctx, **kwargs):
    """Generate or serve the documentation website for your project"""


# dbt docs generate
@docs.command("generate")
@click.pass_context
@p.compile_docs
@p.defer
@p.exclude
@p.log_path
@p.models
@p.profile
@p.profiles_dir
@p.project_dir
@p.selector
@p.state
@p.target
@p.target_path
@p.threads
@p.vars
@p.version_check
def docs_generate(ctx, **kwargs):
    """Generate the documentation website for your project"""
    flags = Flags()
    click.echo(f"`{inspect.stack()[0][3]}` called\n flags: {flags}")


# dbt docs serve
@docs.command("serve")
@click.pass_context
@p.browser
@p.port
@p.profile
@p.profiles_dir
@p.project_dir
@p.target
@p.vars
def docs_serve(ctx, **kwargs):
    """Serve the documentation website for your project"""
    flags = Flags()
    click.echo(f"`{inspect.stack()[0][3]}` called\n flags: {flags}")


# dbt compile
@cli.command("compile")
@click.pass_context
@p.defer
@p.exclude
@p.full_refresh
@p.log_path
@p.models
@p.parse_only
@p.profile
@p.profiles_dir
@p.project_dir
@p.selector
@p.state
@p.target
@p.target_path
@p.threads
@p.vars
@p.version_check
def compile(ctx, **kwargs):
    """Generates executable SQL from source, model, test, and analysis files. Compiled SQL files are written to the target/ directory."""
    flags = Flags()
    click.echo(f"`{inspect.stack()[0][3]}` called\n flags: {flags}")


# dbt debug
@cli.command("debug")
@click.pass_context
@p.config_dir
@p.profile
@p.profiles_dir
@p.project_dir
@p.target
@p.vars
@p.version_check
def debug(ctx, **kwargs):
    """Show some helpful information about dbt for debugging. Not to be confused with the --debug option which increases verbosity."""
    flags = Flags()
    click.echo(f"`{inspect.stack()[0][3]}` called\n flags: {flags}")


# dbt deps
@cli.command("deps")
@click.pass_context
@p.profile
@p.profiles_dir
@p.project_dir
@p.target
@p.vars
def deps(ctx, **kwargs):
    """Pull the most recent version of the dependencies listed in packages.yml"""
    flags = Flags()
    click.echo(f"`{inspect.stack()[0][3]}` called\n flags: {flags}")


# dbt init
@cli.command("init")
@click.pass_context
@p.profile
@p.profiles_dir
@p.project_dir
@p.skip_profile_setup
@p.target
@p.vars
def init(ctx, **kwargs):
    """Initialize a new DBT project."""
    flags = Flags()
    click.echo(f"`{inspect.stack()[0][3]}` called\n flags: {flags}")


# dbt list
@cli.command("list")
@click.pass_context
@p.exclude
@p.indirect_selection
@p.models
@p.output
@p.output_keys
@p.profile
@p.profiles_dir
@p.project_dir
@p.resource_type
@p.selector
@p.state
@p.target
@p.vars
def list(ctx, **kwargs):
    """List the resources in your project"""
    flags = Flags()
    click.echo(f"`{inspect.stack()[0][3]}` called\n flags: {flags}")


# dbt parse
@cli.command("parse")
@click.pass_context
@p.compile_parse
@p.log_path
@p.profile
@p.profiles_dir
@p.project_dir
@p.target
@p.target_path
@p.threads
@p.vars
@p.version_check
@p.write_manifest
def parse(ctx, **kwargs):
    """Parses the project and provides information on performance"""
    flags = Flags()
    click.echo(f"`{inspect.stack()[0][3]}` called\n flags: {flags}")


# dbt run
@cli.command("run")
@click.pass_context
@p.defer
@p.exclude
@p.fail_fast
@p.full_refresh
@p.log_path
@p.models
@p.profile
@p.profiles_dir
@p.project_dir
@p.selector
@p.state
@p.target
@p.target_path
@p.threads
@p.vars
@p.version_check
def run(ctx, **kwargs):
    """Compile SQL and execute against the current target database."""
    flags = Flags()
    click.echo(f"`{inspect.stack()[0][3]}` called\n flags: {flags}")


# dbt run operation
@cli.command("run-operation")
@click.pass_context
@p.args
@p.profile
@p.profiles_dir
@p.project_dir
@p.target
@p.vars
def run_operation(ctx, **kwargs):
    """Run the named macro with any supplied arguments."""
    flags = Flags()
    click.echo(f"`{inspect.stack()[0][3]}` called\n flags: {flags}")


# dbt seed
@cli.command("seed")
@click.pass_context
@p.exclude
@p.full_refresh
@p.log_path
@p.models
@p.profile
@p.profiles_dir
@p.project_dir
@p.selector
@p.show
@p.state
@p.target
@p.target_path
@p.threads
@p.vars
@p.version_check
def seed(ctx, **kwargs):
    """Load data from csv files into your data warehouse."""
    flags = Flags()
    click.echo(f"`{inspect.stack()[0][3]}` called\n flags: {flags}")


# dbt snapshot
@cli.command("snapshot")
@click.pass_context
@p.defer
@p.exclude
@p.models
@p.profile
@p.profiles_dir
@p.project_dir
@p.selector
@p.state
@p.target
@p.threads
@p.vars
def snapshot(ctx, **kwargs):
    """Execute snapshots defined in your project"""
    flags = Flags()
    click.echo(f"`{inspect.stack()[0][3]}` called\n flags: {flags}")


# dbt source
@cli.group()
@click.pass_context
def source(ctx, **kwargs):
    """Manage your project's sources"""


# dbt source freshness
@source.command("freshness")
@click.pass_context
@p.exclude
@p.models
@p.output_path  # TODO: Is this ok to re-use?  We have three different output params, how much can we consolidate?
@p.profile
@p.profiles_dir
@p.project_dir
@p.selector
@p.state
@p.target
@p.threads
@p.vars
def freshness(ctx, **kwargs):
    """Snapshots the current freshness of the project's sources"""
    flags = Flags()
    click.echo(f"`{inspect.stack()[0][3]}` called\n flags: {flags}")


# dbt test
@cli.command("test")
@click.pass_context
@p.defer
@p.exclude
@p.fail_fast
@p.indirect_selection
@p.log_path
@p.models
@p.profile
@p.profiles_dir
@p.project_dir
@p.selector
@p.state
@p.store_failures
@p.target
@p.target_path
@p.threads
@p.vars
@p.version_check
def test(ctx, **kwargs):
    """Runs tests on data in deployed models. Run this after `dbt run`"""
    flags = Flags()
    click.echo(f"`{inspect.stack()[0][3]}` called\n flags: {flags}")


# Support running as a module
if __name__ == "__main__":
    cli_runner()
